      subroutine atxpxa(a,u,c,n,na,nu,nc,eps,fail)
*
* Code for solving Lyapunov and Sylvester equations
* Bartels and Stewart, Comm. of the ACM, vol. 15, pp. 820-826, 1972
*
      integer n,na,nu,nc,fail,n1,nm1,i,j,k
      double precision a(na,n),u(nu,1),c(nc,n)
      double precision eps
c     print*, 'a', ((a(i,j),j=1,n),i=1,n)
c     print*, 'c', ((c(i,j),j=1,n),i=1,n)
*
      n1 = n + 1
      nm1 = n - 1
*
      if (eps .lt. .0) goto 15
      call hshldr(a,n,na)
      call bckmlt(a,u,n,na,nu)
      do 10 i = 1,nm1
        a(i+1,i) = a(i,n1)
10    continue
      call schur(a,u,n,na,nu,eps,fail)
      if (fail .ne. 0) return
*
15    do 20 i = 1,n
        c(i,i) = .5d0 * c(i,i)
20    continue
      do 40 i = 1,n
        do 30 j = 1,n
          a(n1,j) = 0.d0
          do 30 k = i,n
            a(n1,j) = a(n1,j) + c(i,k) * u(k,j)
30      continue
        do 40 j = 1,n
          c(i,j) = a(n1,j)
40     continue
       do 60 j = 1,n
         do 50 i = 1,n
           a(i,n1) = 0.d0
           do 50 k = 1,n
             a(i,n1) = a(i,n1) + u(k,i) * c(k,j)
50       continue
         do 60 i = 1,n
           c(i,j) = a(i,n1)
60     continue
       do 70 i = 1,n
         do 70 j = i,n
           c(i,j) = c(i,j) + c(j,i)
           c(j,i) = c(i,j)
70     continue
*
       call symslv(a,c,n,na,nc)
*
       do 80 i = 1,n
         c(i,i) = .5d0 * c(i,i)
80     continue
       do 100 i = 1,n
         do 90 j = 1,n
           a(n1,j) = 0.d0
           do 90 k = i,n
             a(n1,j) = a(n1,j) + c(i,k) * u(j,k)
90       continue
         do 100 j = 1,n
           c(i,j) = a(n1,j)
100    continue
       do 120 j = 1,n
         do 110 i = 1,n
           a(i,n1) = 0.d0
           do 110 k = 1,n
             a(i,n1) = a(i,n1) + u(i,k) * c(k,j)
110      continue
         do 120 i = 1,n
           c(i,j) = a(i,n1)
120    continue
       do 130 i = 1,n
         do 130 j = i,n
           c(i,j) = c(i,j) + c(j,i)
           c(j,i) = c(i,j)
130    continue
*
      return
      end
c
      subroutine hshldr(a,n,na)
*
* Code for solving Lyapunov and Sylvester equations
* Bartels and Stewart, Comm. of the ACM, vol. 15, pp. 820-826, 1972
*
      integer n,na,nm2,n1,l,l1,i,j
      double precision a(na,1),max,sum,s,p
*
      nm2 = n - 2
      n1 = n + 1
      if (n .eq. 1) return
      if (n .gt. 2) goto 5
      a(1,n1) = a(2,1)
      return
*
5     continue
      do 80 l = 1,nm2
        l1 = l + 1
        max = .0
*
        do 10 i = l1,n
          max = dmax1(max,dabs(a(i,l)))
10      continue
*
        if (max .ne. .0) goto 20
        a(l,n1) = .0
        a(n1,l) = .0
        goto 80
*
20      continue
        sum = .0
        do 30 i = l1,n
          a(i,l) = a(i,l) / max
          sum = sum + a(i,l) ** 2
30      continue
*
        s = sign(sqrt(sum),a(l1,l))
        a(l,n1) = -max * s
        a(l1,l) = s + a(l1,l)
        a(n1,l) = s * a(l1,l)
*
        do 50 j = l1,n
          sum = .0
          do 40 i = l1,n
            sum = sum + a(i,l) * a(i,j)
40        continue
*
          p = sum / a(n1,l)
          do 50 i = l1,n
            a(i,j) = a(i,j) - a(i,l) * p
50      continue
*
        do 70 i = 1,n
          sum = .0
          do 60 j = l1,n
            sum = sum + a(i,j) * a(j,l)
60        continue
*
          p = sum / a(n1,l)
          do 70 j = l1,n
            a(i,j) = a(i,j) - p * a(j,l)
70      continue
*
80    continue
      a(n-1,n1) = a(n,n-1)
*
      return
      end

      subroutine bckmlt(a,u,n,na,nu)
*
* Code for solving Lyapunov and Sylvester equations
* Bartels and Stewart, Comm. of the ACM, vol. 15, pp. 820-826, 1972
*
      integer n,na,nu,n1,nm1,nm2,ll,l,l1,i,j
      double precision a(na,1),u(nu,1),sum,p
*
      n1 = n + 1
      nm1 = n - 1
      nm2 = n - 2
      u(n,n) = 1.
*
      if (nm1 .eq. 0) return
      u(nm1,n) = .0
      u(n,nm1) = .0
      u(nm1,nm1) = 1.
*
      if (nm2 .eq. 0) return
      do 40 ll = 1,nm2
        l = nm2 - ll + 1
        l1 = l + 1
        if (a(n1,l) .eq. .0) goto 25
*
        do 20 j = l1,n
          sum = .0
          do 10 i = l1,n
            sum = sum + a(i,l) * u(i,j)
10        continue
*
          p = sum / a(n1,l)
          do 20 i = l1,n
            u(i,j) = u(i,j) - a(i,l) * p
20      continue
*
25      continue
        do 30 i = l1,n
          u(i,l) = .0
          u(l,i) = .0
30      continue
        u(l,l) = 1.
*
40    continue
*
      return
      end

      subroutine schur(h,u,nn,nh,nu,eps,fail)
*
* Code for solving Lyapunov and Sylvester equations
* Bartels and Stewart, Comm. of the ACM, vol. 15, pp. 820-826, 1972
*
      integer nn,na,nh,fail,i,its,j,jl,k,l,ll,m,mm,m2,m3,n,nu,nm2
      logical last
      double precision h(nh,1),u(nu,1),eps,hn,rsum,test,p,q,r,s,w,x,y,z
*
      n = nn
      hn = .0
*
      do 20 i = 1,n
        jl = max0(1,i-1)
        rsum = .0
*
        do 10 j = jl,n
          rsum = rsum + dabs(h(i,j))
10      continue
        hn = dmax1(hn,rsum)
20    continue
*
      test = eps * hn
      if (hn .eq. .0) goto 230
*
30    continue
      if (n .le. 1) goto 230
      its = 0
      na = n - 1
      nm2 = n - 2
*
40    continue
      do 50 ll = 2,n
        l = n - ll + 2
        if (dabs(h(l,l-1)) .le. test) goto 60
50    continue
*
      l = 1
      goto 70
*
60    continue
      h(l,l-1) = .0
*
70    continue
      if (l .lt. na) goto 72
      n = l - 1
      goto 30
*
72    continue
      x = h(n,n) / hn
      y = h(na,na) / hn
      r = (h(n,na) / hn) * (h(na,n) / hn)
      if (its .lt. 30) goto 75
*
      fail = n
      return
*
75    continue
      if (its .eq. 10 .or. its .eq. 20) goto 80
      s = x + y
      y = x * y - r
      goto 90
*
80    continue
      y = (dabs(h(n,na)) + dabs(h(na,nm2))) / hn
      s = 1.5 * y
      y = y ** 2
*
90    continue
      its = its + 1
*
      do 100 mm = l,nm2
        m = nm2 - mm + l
        x = h(m,m) / hn
        r = h(m+1,m) / hn
        z = h(m+1,m+1) / hn
*
        p = x * (x - s) + y + r * (h(m,m+1) / hn)
        q = r * (x + z - s)
        r = r * (h(m+2,m+1) / hn)
        w = dabs(p) + dabs(q) + dabs(r)
*
        p = p / w
        q = q / w
        r = r / w
        if (m .eq. l) goto 110
        if (dabs(h(m,m-1)) * (dabs(q) + dabs(r)) .le. dabs(p) * test)
     &     goto 110
100   continue
*
110   continue
      m2 = m + 2
      m3 = m + 3
      do 120 i = m2,n
        h(i,i-2) = .0
120   continue
*
      if (m3 .gt. n) goto 140
      do 130 i = m3,n
        h(i,i-3) = .0
130   continue
*
140   continue
      do 220 k = m,na
        last = k .eq. na
        if (k .eq. m) goto 150
*
        p = h(k,k-1)
        q = h(k+1,k-1)
        r = .0
        if (.not. last) r = h(k+2,k-1)
        x = dabs(p) + dabs(q) + dabs(r)
        if (x .eq. .0) goto 220
*
        p = p / x
        q = q / x
        r = r / x
*
150     continue
        s = sqrt(p ** 2 + q ** 2 + r ** 2)
        if (p .lt. .0) s = -s
        if (k .ne. m) h(k,k-1) = -s * x
        if (k .eq. m .and. l .ne. m) h(k,k-1) = -h(k,k-1)
        p = p + s
        x = p / s
        y = q / s
        z = r / s
*
        q = q / p
        r = r / p
*
        do 170 j = k,nn
          p = h(k,j) + q * h(k+1,j)
          if (last) goto 160
          p = p + r * h(k+2,j)
          h(k+2,j) = h(k+2,j) - p * z
160       continue
          h(k+1,j) = h(k+1,j) - p * y
          h(k,j) = h(k,j) - p * x
170     continue
*
        j = min0(k+3,n)
        do 190 i = 1,j
          p = x * h(i,k) + y * h(i,k+1)
          if (last) goto 180
          p = p + z * h(i,k+2)
          h(i,k+2) = h(i,k+2) - p * r
*
180       continue
          h(i,k+1) = h(i,k+1) - p * q
          h(i,k) = h(i,k) - p
190     continue
*
        do 210 i = 1,nn
          p = x * u(i,k) + y * u(i,k+1)
          if (last) goto 200
          p = p + z * u(i,k+2)
          u(i,k+2) = u(i,k+2) - p * r
*
200       continue
          u(i,k+1) = u(i,k+1) - p * q
          u(i,k) = u(i,k) - p
210     continue
*
220   continue
      goto 40
*
230   continue
      fail = 0
*
      return
      end

      subroutine symslv(a,c,n,na,nc)
*
* Code for solving Lyapunov and Sylvester equations
* Bartels and Stewart, Comm. of the ACM, vol. 15, pp. 820-826, 1972
*
      integer n,na,nc,k,kk,dk,km1,l,ll,dl,ldl,i,ia,j,nsys
      double precision a(na,1),c(nc,1)
      double precision t,p
      common /slvblk/ t(5,5),p(5),nsys
*
      l = 1
10    dl = 1
      if (l .eq. n) goto 20
      if (a(l+1,l) .ne. .0) dl = 2
20    ll = l + dl - 1
      k = l
30    km1 = k - 1
      dk = 1
      if (k .eq. n) goto 35
      if (a(k+1,k) .ne .0) dk = 2
35    kk = k + dk - 1
      if (k .eq. l) goto 45
      do 40 i = k,kk
        do 40 j = l,ll
          do 40 ia = l,km1
            c(i,j) = c(i,j) - a(ia,i) * c(ia,j)
40    continue
45    if (dl .eq. 2) goto 60
      if (dk .eq. 2) goto 50
      t(1,1) = a(k,k) + a(l,l)
      if (t(1,1) .eq. .0) stop
      c(k,l) = c(k,l) / t(1,1)
      goto 90
50    t(1,1) = a(k,k) + a(l,l)
      t(1,2) = a(kk,k)
      t(2,1) = a(k,kk)
      t(2,2) = a(kk,kk) + a(l,l)
      p(1) = c(k,l)
      p(2) = c(kk,l)
      nsys = 2
      call sysslv
      c(k,l) = p(1)
      c(kk,l) = p(2)
      goto 90
60    if (dk .eq. 2) goto 70
      t(1,1) = a(k,k) + a(l,l)
      t(1,2) = a(ll,l)
      t(2,1) = a(l,ll)
      t(2,2) = a(k,k) + a(ll,ll)
      p(1) = c(k,l)
      p(2) = c(k,ll)
      nsys = 2
      call sysslv
      c(k,l) = p(1)
      c(k,ll) = p(2)
      goto 90
70    if (k .ne. l) goto 80
      t(1,1) = a(l,l)
      t(1,2) = a(ll,l)
      t(1,3) = 0.d0
      t(2,1) = a(l,ll)
      t(2,2) = a(l,l) + a(ll,ll)
      t(2,3) = t(1,2)
      t(3,1) = 0.d0
      t(3,2) = t(2,1)
      t(3,3) = a(ll,ll)
      p(1) = .5d0 * c(l,l)
      p(2) = c(ll,l)
      p(3) = .5d0 * c(ll,ll)
      nsys = 3
      call sysslv
      c(l,l) = p(1)
      c(ll,l) = p(2)
      c(l,ll) = p(2)
      c(ll,ll) = p(3)
      goto 90
80    t(1,1) = a(k,k) + a(l,l)
      t(1,2) = a(kk,k)
      t(1,3) = a(ll,l)
      t(1,4) = 0.d0
      t(2,1) = a(k,kk)
      t(2,2) = a(kk,kk) + a(l,l)
      t(2,3) = 0.d0
      t(2,4) = t(1,3)
      t(3,1) = a(l,ll)
      t(3,2) = 0.d0
      t(3,3) = a(k,k) + a(ll,ll)
      t(3,4) = t(1,2)
      t(4,1) = 0.d0
      t(4,2) = t(3,1)
      t(4,3) = t(2,1)
      t(4,4) = a(kk,kk) + a(ll,ll)
      p(1) = c(k,l)
      p(2) = c(kk,l)
      p(3) = c(k,ll)
      p(4) = c(kk,ll)
      nsys = 4
      call sysslv
      c(k,l) = p(1)
      c(kk,l) = p(2)
      c(k,ll) = p(3)
      c(kk,ll) = p(4)
90    k = k + dk
      if (k .le. n) goto 30
      ldl = l + dl
      if (ldl .gt. n) return
      do 120 j = ldl,n
        do 100 i = l,ll
          c(i,j) = c(j,i)
100     continue
        do 120 i = j,n
          do 110 k = l,ll
            c(i,j) = c(i,j) - c(i,k) * a(k,j) - a(k,i) * c(k,j)
110       continue
          c(j,i) = c(i,j)
120   continue
      l = ldl
      goto 10
*
      end

      subroutine sysslv()
*
* Code for solving Lyapunov and Sylvester equations
* Bartels and Stewart, Comm. of the ACM, vol. 15, pp. 820-826, 1972
*
      common /slvblk/ a(5,5),b(5),n
*
      integer n,nm1,n1,k,km1,i,j,kp1,intr,im1,ii,i1
      double precision a,aa,b,max,temp
*
      nm1 = n - 1
      n1 = n + 1
*
      do 80 k = 1,n
        km1 = k - 1
        if (k .eq. 1) goto 20
        do 10 i = k,n
          do 10 j = 1,km1
            a(i,k) = a(i,k) - a(i,j) * a(j,k)
10      continue
*
20      continue
        if (k .eq. n) goto 100
        kp1 = k + 1
        max = dabs(a(k,k))
        intr = k
*
        do 30 i = kp1,n
          aa = dabs(a(i,k))
          if (aa .le. max) goto 30
          max = aa
          intr = i
30      continue
*
        if (max .eq. .0) stop
        a(n1,k) = intr
        if (intr .eq. k) goto 50
*
        do 40 j = 1,n
          temp = a(k,j)
          a(k,j) = a(intr,j)
          a(intr,j) = temp
40      continue
*
50      continue
        do 80 j = kp1,n
          if (k .eq. 1) goto 70
          do 60 i = 1,km1
            a(k,j) = a(k,j) - a(k,i) * a(i,j)
60        continue
*
70        continue
          a(k,j) = a(k,j) / a(k,k)
80    continue
*
100   continue
      do 110 j = 1,nm1
        intr = a(n1,j)
        if (intr .eq. j) goto 110
        temp = b(j)
        b(j) = b(intr)
        b(intr) = temp
110   continue
*
200   continue
      b(1) = b(1) / a(1,1)
      do 220 i = 2,n
        im1 = i - 1
        do 210 j = 1,im1
          b(i) = b(i) - a(i,j) * b(j)
210     continue
        b(i) = b(i) / a(i,i)
220   continue
*
300   continue
      do 310 ii = 1,nm1
        i = nm1 - ii + 1
        i1 = i + 1
        do 310 j = i1,n
          b(i) = b(i) - a(i,j) * b(j)
310   continue
*
      return
      end
