      SUBROUTINE ROOT(T,FT,B,C,RELERR,ABSERR,IFLAG)
C
C  ROOT COMPUTES A ROOT OF THE NONLINEAR EQUATION F(X)=0
C  WHERE F(X) IS A CONTINOUS REAL FUNCTION OF A SINGLE REAL
C  VARIABLE X.  THE METHOD USED IS A COMBINATION OF BISECTION
C  AND THE SECANT RULE.
C
C  NORMAL INPUT CONSISTS OF A CONTINUOS FUNCTION F AND AN
C  INTERVAL (B,C) SUCH THAT F(B)*F(C).LE.0.0.  EACH ITERATION
C  FINDS NEW VALUES OF B AND C SUCH THAT THE INTERVAL(B,C) IS
C  SHRUNK AND F(B)*F(C).LE.0.0.  THE STOPPING CRITERION IS
C
C          DABS(B-C).LE.2.0*(RELERR*DABS(B)+ABSERR)
C
C  WHERE RELERR=RELATIVE ERROR AND ABSERR=ABSOLUTE ERROR ARE
C  INPUT QUANTITIES.  SET THE FLAG, IFLAG, POSITIVE TO INITIALIZE
C  THE COMPUTATION.  AS B,C AND IFLAG ARE USED FOR BOTH INPUT AND
C  OUTPUT, THEY MUST BE VARIABLES IN THE CALLING PROGRAM.
C
C  IF 0 IS A POSSIBLE ROOT, ONE SHOULD NOT CHOOSE ABSERR=0.0.
C
C  THE OUTPUT VALUE OF B IS THE BETTER APPROXIMATION TO A ROOT
C  AS B AND C ARE ALWAYS REDEFINED SO THAT DABS(F(B)).LE.DABS(F(C)).
C
C  TO SOLVE THE EQUATION, ROOT MUST EVALUATE F(X) REPEATEDLY. THIS
C  IS DONE IN THE CALLING PROGRAM.  WHEN AN EVALUATION OF F IS
C  NEEDED AT T, ROOT RETURNS WITH IFLAG NEGATIVE.  EVALUATE FT=F(T)
C  AND CALL ROOT AGAIN.  DO NOT ALTER IFLAG.
C
C  WHEN THE COMPUTATION IS COMPLETE, ROOT RETURNS TO THE CALLING
C  PROGRAM WITH IFLAG POSITIVE=
C
C     IFLAG=1  IF F(B)*F(C).LT.0 AND THE STOPPING CRITERION IS MET.
C
C          =2  IF A VALUE B IS FOUND SUCH THAT THE COMPUTED VALUE
C              F(B) IS EXACTLY ZERO.  THE INTERVAL (B,C) MAY NOT
C              SATISFY THE STOPPING CRITERION.
C
C          =3  IF DABS(F(B)) EXCEEDS THE INPUT VALUES DABS(F(B)),
C              DABS(F(C)).  IN THIS CASE IT IS LIKELY THAT B IS CLOSE
C              TO A POLE OF F.
C
C          =4  IF NO ODD ORDER ROOT WAS FOUND IN THE INTERVAL.  A
C              LOCAL MINIMUM MAY HAVE BEEN OBTAINED.
C
C          =5  IF TOO MANY FUNCTION EVALUATIONS WERE MADE.
C              (AS PROGRAMMED, 500 ARE ALLOWED.)
C
C  THIS CODE IS A MODIFICATION OF THE CODE ZEROIN WHICH IS COMPLETELY
C  EXPLAINED AND DOCUMENTED IN THE TEXT  NUMERICAL COMPUTING:  AN
C  INTRODUCTION,  BY L. F. SHAMPINE AND R. C. ALLEN.
C
C  CALLS  D1MACH .
C
      DOUBLE PRECISION A,ABSERR,ACBS,ACMB,AE,B,C,CMB,D1MACH,FA,FB,
     1  FC,FT,FX,P,Q,RE,RELERR,T,TOL,U
      INTEGER IC,IFLAG,KOUNT
      SAVE
C
      IF(IFLAG.GE.0) GO TO 100
      IFLAG=ABS(IFLAG)
      GO TO (200,300,400), IFLAG
  100 U=D1MACH(4)
      RE=MAX(RELERR,U)
      AE=MAX(ABSERR,0.0D0)
      IC=0
      ACBS=ABS(B-C)
      A=C
      T=A
      IFLAG=-1
      RETURN
  200 FA=FT
      T=B
      IFLAG=-2
      RETURN
  300 FB=FT
      FC=FA
      KOUNT=2
      FX=MAX(ABS(FB),ABS(FC))
    1 IF(ABS(FC).GE.ABS(FB))GO TO 2
C
C  INTERCHANGE B AND C SO THAT ABS(F(B)).LE.ABS(F(C)).
C
      A=B
      FA=FB
      B=C
      FB=FC
      C=A
      FC=FA
    2 CMB=0.5*(C-B)
      ACMB=ABS(CMB)
      TOL=RE*ABS(B)+AE
C
C  TEST STOPPING CRITERION AND FUNCTION COUNT.
C
      IF(ACMB.LE.TOL)GO TO 8
      IF(KOUNT.GE.500)GO TO 12
C
C  CALCULATE NEW ITERATE EXPLICITLY AS B+P/Q
C  WHERE WE ARRANGE P.GE.0.  THE IMPLICIT
C  FORM IS USED TO PREVENT OVERFLOW.
C
      P=(B-A)*FB
      Q=FA-FB
      IF(P.GE.0.0)GO TO 3
      P=-P
      Q=-Q
C
C  UPDATE A, CHECK IF REDUCTION IN THE SIZE OF BRACKETING
C  INTERVAL IS SATISFACTORY.  IF NOT BISECT UNTIL IT IS.
C
    3 A=B
      FA=FB
      IC=IC+1
      IF(IC.LT.4)GO TO 4
      IF(8.0*ACMB.GE.ACBS)GO TO 6
      IC=0
      ACBS=ACMB
C
C  TEST FOR TOO SMALL A CHANGE.
C
    4 IF(P.GT.ABS(Q)*TOL)GO TO 5
C
C  INCREMENT BY TOLERANCE
C
      B=B+SIGN(TOL,CMB)
      GO TO 7
C
C  ROOT OUGHT TO BE BETWEEN B AND (C+B)/2
C
    5 IF(P.GE.CMB*Q)GO TO 6
C
C  USE SECANT RULE.
C
      B=B+P/Q
      GO TO 7
C
C  USE BISECTION.
C
    6 B=0.5*(C+B)
C
C  HAVE COMPLETED COMPUTATION FOR NEW ITERATE B.
C
    7 T=B
      IFLAG=-3
      RETURN
  400 FB=FT
      IF(FB.EQ.0.0)GO TO 9
      KOUNT=KOUNT+1
      IF(SIGN(1.0D0,FB).NE.SIGN(1.0D0,FC))GO TO 1
      C=A
      FC=FA
      GO TO 1
C
C FINISHED.  SET IFLAG.
C
    8 IF(SIGN(1.0D0,FB).EQ.SIGN(1.0D0,FC))GO TO 11
      IF(ABS(FB).GT.FX)GO TO 10
      IFLAG=1
      RETURN
    9 IFLAG=2
      RETURN
   10 IFLAG=3
      RETURN
   11 IFLAG=4
      RETURN
   12 IFLAG=5
      RETURN
      END
      subroutine rootnf(n,nfe,iflag,relerr,abserr,y,yp,yold,ypold,
     &   a,qr,alpha,tz,pivot,w,wp,par,ipar)
c
c rootnf  finds the point  ybar = (1, xbar)  on the zero curve of the
c homotopy map.  it starts with two points yold=(lambdaold,xold) and
c y=(lambda,x) such that  lambdaold < 1 <= lambda , and alternates
c between hermite cubic interpolation and newton iteration until
c convergence.
c
c on input:
c
c n = dimension of x and the homotopy map.
c
c nfe = number of jacobian matrix evaluations.
c
c iflag = -2, -1, or 0, indicating the problem type.
c
c relerr, abserr = relative and absolute error values.  the iteration is
c    considered to have converged when a point y=(lambda,x) is found 
c    such that
c
c    |y(1) - 1| <= relerr + abserr              and
c
c    ||z|| <= relerr*||x|| + abserr  ,          where
c
c    (?,z) is the newton step to y=(lambda,x).
c
c y(1:n+1) = point (lambda(s), x(s)) on zero curve of homotopy map.
c
c yp(1:n+1) = unit tangent vector to the zero curve of the homotopy map
c    at  y .
c
c yold(1:n+1) = a point different from  y  on the zero curve.
c
c ypold(1:n+1) = unit tangent vector to the zero curve of the homotopy
c    map at  yold .
c
c a(1:*) = parameter vector in the homotopy map.
c
c qr(1:n,1:n+2), alpha(1:n), tz(1:n+1), pivot(1:n+1), w(1:n+1), 
c    wp(1:n+1)  are work arrays used for the qr factorization (in the
c    newton step calculation) and the interpolation.
c
c par(1:*) and ipar(1:*) are arrays for (optional) user parameters,
c    which are simply passed through to the user written subroutines
c    rho, rhojac.
c
c on output:
c
c n , relerr , abserr , a  are unchanged.
c
c nfe  has been updated.
c
c iflag 
c    = -2, -1, or 0 (unchanged) on a normal return.
c
c    = 4 if a jacobian matrix with rank < n has occurred.  the
c        iteration was not completed.
c
c    = 6 if the iteration failed to converge.  y  and  yold  contain
c        the last two points found on the zero curve.
c
c y  is the point on the zero curve of the homotopy map at  lambda = 1 .
c
c
c calls  d1mach , dnrm2 , root , tangnf .
c
      double precision abserr,aerr,d1mach,dd001,dd0011,dd01,dd011,
     &   dels,dnrm2,f0,f1,fp0,fp1,qofs,qsout,relerr,rerr,s,sa,sb,
     &   sout,u
      integer iflag,judy,jw,lcode,limit,n,nfe,np1
c
c ***** array declarations. *****
c
      double precision y(n+1),yp(n+1),yold(n+1),ypold(n+1),a(n),
     &   qr(n,n+2),alpha(n),tz(n+1),w(n+1),wp(n+1),par(1)
      integer pivot(n+1),ipar(1)
c
c ***** end of dimensional information. *****
c
c the limit on the number of iterations allowed may be changed by
c changing the following parameter statement:
      parameter (limit=20)
c
c definition of hermite cubic interpolant via divided differences.
c
      dd01(f0,f1,dels)=(f1-f0)/dels
      dd001(f0,fp0,f1,dels)=(dd01(f0,f1,dels)-fp0)/dels
      dd011(f0,f1,fp1,dels)=(fp1-dd01(f0,f1,dels))/dels
      dd0011(f0,fp0,f1,fp1,dels)=(dd011(f0,f1,fp1,dels) - 
     &                            dd001(f0,fp0,f1,dels))/dels
      qofs(f0,fp0,f1,fp1,dels,s)=((dd0011(f0,fp0,f1,fp1,dels)*(s-dels) +
     &   dd001(f0,fp0,f1,dels))*s + fp0)*s + f0
c
c
      u=d1mach(4)
      rerr=max(relerr,u)
      aerr=max(abserr,0.0d0)
      np1=n+1
c
c *****  main loop.  *****
c
100   do 300 judy=1,limit
      do 110 jw=1,np1
        tz(jw)=y(jw)-yold(jw)
110   continue
      dels=dnrm2(np1,tz,1)
c     write(6,303) y(1),yold(1),dels
c303   format(/,d25.15,2x,d25.15,2x,d25.15)
c
c using two points and tangents on the homotopy zero curve, construct 
c the hermite cubic interpolant q(s).  then use  root  to find the s
c corresponding to  lambda = 1 .  the two points on the zero curve are
c always chosen to bracket lambda=1, with the bracketing interval
c always being [0, dels].
c
      sa=0.0
      sb=dels
      lcode=1
130   call root(sout,qsout,sa,sb,rerr,aerr,lcode)
      if (lcode .gt. 0) go to 140
      qsout=qofs(yold(1),ypold(1),y(1),yp(1),dels,sout) - 1.0
      go to 130
c if lambda = 1 were bracketed,  root  cannot fail.
140   if (lcode .gt. 2) then
        print*,'lcode',lcode
        iflag=6
        return
      endif
c
c calculate q(sa) as the initial point for a newton iteration.
      do 150 jw=1,np1
        w(jw)=qofs(yold(jw),ypold(jw),y(jw),yp(jw),dels,sa)
150   continue
c calculate newton step at q(sa).
      call tangnf(sa,w,wp,ypold,a,qr,alpha,tz,pivot,nfe,n,iflag,
     &            par,ipar)
      if (iflag .gt. 0) return
c next point = current point + newton step.
      do 160 jw=1,np1
        w(jw)=w(jw)+tz(jw)
160   continue
c get the tangent  wp  at  w  and the next newton step in  tz .
      call tangnf(sa,w,wp,ypold,a,qr,alpha,tz,pivot,nfe,n,iflag,
     &            par,ipar)
      if (iflag .gt. 0) return
c take newton step and check convergence.
      do 170 jw=1,np1
        w(jw)=w(jw)+tz(jw)
170   continue
      if ((abs(w(1)-1.0) .le. rerr+aerr) .and.
     &    (dnrm2(n,tz(2),1) .le. rerr*dnrm2(n,w(2),1)+aerr)) then
        do 180 jw=1,np1
          y(jw)=w(jw)
180     continue
        return
      endif
c if the iteration has not converged, discard one of the old points
c such that  lambda = 1  is still bracketed.
      if ((yold(1)-1.0)*(w(1)-1.0) .gt. 0.0) then
        do 200 jw=1,np1
          yold(jw)=w(jw)
          ypold(jw)=wp(jw)
200     continue
      else
        do 210 jw=1,np1
          y(jw)=w(jw)
          yp(jw)=wp(jw)
210     continue
      endif
300   continue
c
c ***** end of main loop. *****
c
c the alternating osculatory cubic interpolation and newton iteration
c has not converged in  limit  steps.  error return.
      iflag=6
      return
      end
      SUBROUTINE STEPNF(N,NFE,IFLAG,START,CRASH,HOLD,H,RELERR,
     &   ABSERR,S,Y,YP,YOLD,YPOLD,A,QR,ALPHA,TZ,PIVOT,W,WP,
     &   Z0,Z1,SSPAR,PAR,IPAR)
C
C  STEPNF  TAKES ONE STEP ALONG THE ZERO CURVE OF THE HOMOTOPY MAP
C USING A PREDICTOR-CORRECTOR ALGORITHM.  THE PREDICTOR USES A HERMITE
C CUBIC INTERPOLANT, AND THE CORRECTOR RETURNS TO THE ZERO CURVE ALONG
C THE FLOW NORMAL TO THE DAVIDENKO FLOW.  STEPNF  ALSO ESTIMATES A
C STEP SIZE H FOR THE NEXT STEP ALONG THE ZERO CURVE.  NORMALLY
C  STEPNF  IS USED INDIRECTLY THROUGH  FIXPNF , AND SHOULD BE CALLED
C DIRECTLY ONLY IF IT IS NECESSARY TO MODIFY THE STEPPING ALGORITHM'S
C PARAMETERS.
C
C ON INPUT:
C
C N = DIMENSION OF X AND THE HOMOTOPY MAP.
C
C NFE = NUMBER OF JACOBIAN MATRIX EVALUATIONS.
C
C IFLAG = -2, -1, OR 0, INDICATING THE PROBLEM TYPE.
C
C START = .TRUE. ON FIRST CALL TO  STEPNF , .FALSE. OTHERWISE.
C
C HOLD = ||Y - YOLD||; SHOULD NOT BE MODIFIED BY THE USER.
C
C H = UPPER LIMIT ON LENGTH OF STEP THAT WILL BE ATTEMPTED.  H  MUST BE
C    SET TO A POSITIVE NUMBER ON THE FIRST CALL TO  STEPNF .
C    THEREAFTER  STEPNF  CALCULATES AN OPTIMAL VALUE FOR  H , AND  H
C    SHOULD NOT BE MODIFIED BY THE USER.
C
C RELERR, ABSERR = RELATIVE AND ABSOLUTE ERROR VALUES.  THE ITERATION IS
C    CONSIDERED TO HAVE CONVERGED WHEN A POINT W=(LAMBDA,X) IS FOUND 
C    SUCH THAT
C
C    ||Z|| <= RELERR*||W|| + ABSERR  ,          WHERE
C
C    Z IS THE NEWTON STEP TO W=(LAMBDA,X).
C
C S = (APPROXIMATE) ARC LENGTH ALONG THE HOMOTOPY ZERO CURVE UP TO
C    Y(S) = (LAMBDA(S), X(S)).
C
C Y(1:N+1) = PREVIOUS POINT (LAMBDA(S), X(S)) FOUND ON THE ZERO CURVE OF 
C    THE HOMOTOPY MAP.
C
C YP(1:N+1) = UNIT TANGENT VECTOR TO THE ZERO CURVE OF THE HOMOTOPY MAP
C    AT  Y .
C
C YOLD(1:N+1) = A POINT BEFORE  Y  ON THE ZERO CURVE OF THE HOMOTOPY MAP.
C
C YPOLD(1:N+1) = UNIT TANGENT VECTOR TO THE ZERO CURVE OF THE HOMOTOPY
C    MAP AT  YOLD .
C
C A(1:*) = PARAMETER VECTOR IN THE HOMOTOPY MAP.
C
C QR(1:N,1:N+2), ALPHA(1:N), TZ(1:N+1), PIVOT(1:N+1), W(1:N+1), 
C    WP(1:N+1)  ARE WORK ARRAYS USED FOR THE QR FACTORIZATION (IN THE
C    NEWTON STEP CALCULATION) AND THE INTERPOLATION.
C
C Z0(1:N+1), Z1(1:N+1)  ARE WORK ARRAYS USED FOR THE ESTIMATION OF THE
C    NEXT STEP SIZE  H .
C
C SSPAR(1:8) = (LIDEAL, RIDEAL, DIDEAL, HMIN, HMAX, BMIN, BMAX, P)  IS
C    A VECTOR OF PARAMETERS USED FOR THE OPTIMAL STEP SIZE ESTIMATION.
C
C PAR(1:*) AND IPAR(1:*) ARE ARRAYS FOR (OPTIONAL) USER PARAMETERS,
C    WHICH ARE SIMPLY PASSED THROUGH TO THE USER WRITTEN SUBROUTINES
C    RHO, RHOJAC.
C
C ON OUTPUT:
C
C N , A , SSPAR  ARE UNCHANGED.
C
C NFE  HAS BEEN UPDATED.
C
C IFLAG  
C    = -2, -1, OR 0 (UNCHANGED) ON A NORMAL RETURN.
C
C    = 4 IF A JACOBIAN MATRIX WITH RANK < N HAS OCCURRED.  THE
C        ITERATION WAS NOT COMPLETED.
C
C    = 6 IF THE ITERATION FAILED TO CONVERGE.  W  CONTAINS THE LAST
C        NEWTON ITERATE.
C
C START = .FALSE. ON A NORMAL RETURN.
C
C CRASH 
C    = .FALSE. ON A NORMAL RETURN.
C
C    = .TRUE. IF THE STEP SIZE  H  WAS TOO SMALL.  H  HAS BEEN
C      INCREASED TO AN ACCEPTABLE VALUE, WITH WHICH  STEPNF  MAY BE
C      CALLED AGAIN.
C
C    = .TRUE. IF  RELERR  AND/OR  ABSERR  WERE TOO SMALL.  THEY HAVE
C      BEEN INCREASED TO ACCEPTABLE VALUES, WITH WHICH  STEPNF  MAY
C      BE CALLED AGAIN.
C
C HOLD = ||Y - YOLD||.
C
C H = OPTIMAL VALUE FOR NEXT STEP TO BE ATTEMPTED.  NORMALLY  H  SHOULD
C    NOT BE MODIFIED BY THE USER.
C
C RELERR, ABSERR  ARE UNCHANGED ON A NORMAL RETURN.
C
C S = (APPROXIMATE) ARC LENGTH ALONG THE ZERO CURVE OF THE HOMOTOPY MAP 
C    UP TO THE LATEST POINT FOUND, WHICH IS RETURNED IN  Y .
C
C Y, YP, YOLD, YPOLD  CONTAIN THE TWO MOST RECENT POINTS AND TANGENT
C    VECTORS FOUND ON THE ZERO CURVE OF THE HOMOTOPY MAP.
C
C
C CALLS  D1MACH , DNRM2 , TANGNF .
C
      DOUBLE PRECISION ABSERR,D1MACH,DCALC,DD001,DD0011,DD01,
     &   DD011,DELS,DNRM2,F0,F1,FOURU,FP0,FP1,H,HFAIL,HOLD,HT,
     &   LCALC,QOFS,RCALC,RELERR,RHOLEN,S,TEMP,TWOU
      INTEGER IFLAG,ITNUM,J,JUDY,LITFH,N,NFE,NP1
      LOGICAL CRASH,FAIL,START
C
C ***** ARRAY DECLARATIONS. *****
C
      DOUBLE PRECISION Y(N+1),YP(N+1),YOLD(N+1),YPOLD(N+1),A(N),
     &   QR(N,N+2),ALPHA(N),TZ(N+1),W(N+1),WP(N+1),Z0(N+1),
     &   Z1(N+1),SSPAR(8),PAR(1)
      INTEGER PIVOT(N+1),IPAR(1)
C
C ***** END OF DIMENSIONAL INFORMATION. *****
C
C THE LIMIT ON THE NUMBER OF NEWTON ITERATIONS ALLOWED BEFORE REDUCING
C THE STEP SIZE  H  MAY BE CHANGED BY CHANGING THE FOLLOWING PARAMETER 
C STATEMENT:
      PARAMETER (LITFH=4)
C
C DEFINITION OF HERMITE CUBIC INTERPOLANT VIA DIVIDED DIFFERENCES.
C
      DD01(F0,F1,DELS)=(F1-F0)/DELS
      DD001(F0,FP0,F1,DELS)=(DD01(F0,F1,DELS)-FP0)/DELS
      DD011(F0,F1,FP1,DELS)=(FP1-DD01(F0,F1,DELS))/DELS
      DD0011(F0,FP0,F1,FP1,DELS)=(DD011(F0,F1,FP1,DELS) - 
     &                            DD001(F0,FP0,F1,DELS))/DELS
      QOFS(F0,FP0,F1,FP1,DELS,S)=((DD0011(F0,FP0,F1,FP1,DELS)*(S-DELS) +
     &   DD001(F0,FP0,F1,DELS))*S + FP0)*S + F0
C
C
      TWOU=2.0*D1MACH(4)
      FOURU=TWOU+TWOU
      NP1=N+1
      CRASH=.TRUE.
C THE ARCLENGTH  S  MUST BE NONNEGATIVE.
      IF (S .LT. 0.0) RETURN
C IF STEP SIZE IS TOO SMALL, DETERMINE AN ACCEPTABLE ONE.
      IF (H .LT. FOURU*(1.0+S)) THEN
        H=FOURU*(1.0+S)
        RETURN
      ENDIF
C IF ERROR TOLERANCES ARE TOO SMALL, INCREASE THEM TO ACCEPTABLE VALUES.
      TEMP=DNRM2(NP1,Y,1)
      IF (.5*(RELERR*TEMP+ABSERR) .GE. TWOU*TEMP) GO TO 40
      IF (RELERR .NE. 0.0) THEN
        RELERR=FOURU*(1.0+FOURU)
        ABSERR=MAX(ABSERR,0.0D0)
      ELSE
        ABSERR=FOURU*TEMP
      ENDIF
      RETURN
 40   CRASH=.FALSE.
      IF (.NOT. START) GO TO 300
C
C *****  STARTUP SECTION(FIRST STEP ALONG ZERO CURVE.  *****
C
      FAIL=.FALSE.
      START=.FALSE.
C DETERMINE SUITABLE INITIAL STEP SIZE.
      H=MIN(H, .10D0, SQRT(SQRT(RELERR*TEMP+ABSERR)))
C USE LINEAR PREDICTOR ALONG TANGENT DIRECTION TO START NEWTON ITERATION.
      YPOLD(1)=1.0
      DO 50 J=2,NP1
        YPOLD(J)=0.0
 50   CONTINUE
      CALL TANGNF(S,Y,YP,YPOLD,A,QR,ALPHA,TZ,PIVOT,NFE,N,IFLAG,
     &            PAR,IPAR)
      IF (IFLAG .GT. 0) RETURN
 70   DO 80 J=1,NP1
        TEMP=Y(J) + H * YP(J)
        W(J)=TEMP
        Z0(J)=TEMP
 80   CONTINUE
      DO 200 JUDY=1,LITFH
        RHOLEN=-1.0
C CALCULATE THE NEWTON STEP  TZ  AT THE CURRENT POINT  W .
        CALL TANGNF(RHOLEN,W,WP,YPOLD,A,QR,ALPHA,TZ,PIVOT,NFE,N,IFLAG,
     &              PAR,IPAR)
        IF (IFLAG .GT. 0) RETURN
C
C TAKE NEWTON STEP AND CHECK CONVERGENCE.
        DO 90 J=1,NP1
          W(J)=W(J) + TZ(J)
 90     CONTINUE
        ITNUM=JUDY
C COMPUTE QUANTITIES USED FOR OPTIMAL STEP SIZE ESTIMATION.
        IF (JUDY .EQ. 1) THEN
          LCALC=DNRM2(NP1,TZ,1)
          RCALC=RHOLEN
          DO 110 J=1,NP1
            Z1(J)=W(J)
110       CONTINUE
        ELSE IF (JUDY .EQ. 2) THEN
          LCALC=DNRM2(NP1,TZ,1)/LCALC
          RCALC=RHOLEN/RCALC
        ENDIF
C GO TO MOP-UP SECTION AFTER CONVERGENCE.
        IF (DNRM2(NP1,TZ,1) .LE. RELERR*DNRM2(NP1,W,1)+ABSERR)
     &                                                 GO TO 600
C
200   CONTINUE
C
C NO CONVERGENCE IN  LITFH  ITERATIONS.  REDUCE  H  AND TRY AGAIN.
      IF (H .LE. FOURU*(1.0 + S)) THEN
        IFLAG=6
        RETURN
      ENDIF
      H=.5 * H
      GO TO 70
C
C ***** END OF STARTUP SECTION. *****
C
C ***** PREDICTOR SECTION. *****
C
300   FAIL=.FALSE.
C COMPUTE POINT PREDICTED BY HERMITE INTERPOLANT.  USE STEP SIZE  H
C COMPUTED ON LAST CALL TO  STEPNF .
320   DO 330 J=1,NP1
        TEMP=QOFS(YOLD(J),YPOLD(J),Y(J),YP(J),HOLD,HOLD+H)
        W(J)=TEMP
        Z0(J)=TEMP
330   CONTINUE
C
C ***** END OF PREDICTOR SECTION. *****
C
C ***** CORRECTOR SECTION. *****
C
      DO 500 JUDY=1,LITFH
        RHOLEN=-1.0
C CALCULATE THE NEWTON STEP  TZ  AT THE CURRENT POINT  W .
        CALL TANGNF(RHOLEN,W,WP,YP,A,QR,ALPHA,TZ,PIVOT,NFE,N,IFLAG,
     &              PAR,IPAR)
        IF (IFLAG .GT. 0) RETURN
C
C TAKE NEWTON STEP AND CHECK CONVERGENCE.
        DO 420 J=1,NP1
           W(J)=W(J) + TZ(J)
420     CONTINUE
        ITNUM=JUDY
C COMPUTE QUANTITIES USED FOR OPTIMAL STEP SIZE ESTIMATION.
        IF (JUDY .EQ. 1) THEN
          LCALC=DNRM2(NP1,TZ,1)
          RCALC=RHOLEN
          DO 440 J=1,NP1
            Z1(J)=W(J)
440       CONTINUE
        ELSE IF (JUDY .EQ. 2) THEN
          LCALC=DNRM2(NP1,TZ,1)/LCALC
          RCALC=RHOLEN/RCALC
        ENDIF
C GO TO MOP-UP SECTION AFTER CONVERGENCE.
        IF (DNRM2(NP1,TZ,1) .LE. RELERR*DNRM2(NP1,W,1)+ABSERR)
     &                                                 GO TO 600
C
500   CONTINUE
C
C NO CONVERGENCE IN  LITFH  ITERATIONS.  RECORD FAILURE AT CALCULATED  H , 
C SAVE THIS STEP SIZE, REDUCE  H  AND TRY AGAIN.
      FAIL=.TRUE.
      HFAIL=H
      IF (H .LE. FOURU*(1.0 + S)) THEN
        IFLAG=6
        RETURN
      ENDIF
      H=.5 * H
      GO TO 320
C
C ***** END OF CORRECTOR SECTION. *****
C
C ***** MOP-UP SECTION. *****
C
C YOLD  AND  Y  ALWAYS CONTAIN THE LAST TWO POINTS FOUND ON THE ZERO
C CURVE OF THE HOMOTOPY MAP.  YPOLD  AND  YP  CONTAIN THE TANGENT
C VECTORS TO THE ZERO CURVE AT  YOLD  AND  Y , RESPECTIVELY.
C
600   DO 620 J=1,NP1
        YOLD(J)=Y(J)
        YPOLD(J)=YP(J)
        Y(J)=W(J)
        YP(J)=WP(J)
        W(J)=Y(J) - YOLD(J)
620   CONTINUE
C UPDATE ARC LENGTH.
      HOLD=DNRM2(NP1,W,1)
      S=S+HOLD
C
C ***** END OF MOP-UP SECTION. *****
C
C ***** OPTIMAL STEP SIZE ESTIMATION SECTION. *****
C
C CALCULATE THE DISTANCE FACTOR  DCALC .
700   DO 710 J=1,NP1
        TZ(J)=Z0(J) - Y(J)
        W(J)=Z1(J) - Y(J)
710   CONTINUE
      DCALC=DNRM2(NP1,TZ,1)
      IF (DCALC .NE. 0.0) DCALC=DNRM2(NP1,W,1)/DCALC
C
C THE OPTIMAL STEP SIZE HBAR IS DEFINED BY
C
C   HT=HOLD * [MIN(LIDEAL/LCALC, RIDEAL/RCALC, DIDEAL/DCALC)]**(1/P)
C
C     HBAR = MIN [ MAX(HT, BMIN*HOLD, HMIN), BMAX*HOLD, HMAX ]
C
C IF CONVERGENCE HAD OCCURRED AFTER 1 ITERATION, SET THE CONTRACTION
C FACTOR  LCALC  TO ZERO.
      IF (ITNUM .EQ. 1) LCALC = 0.0
C FORMULA FOR OPTIMAL STEP SIZE.
      IF (LCALC+RCALC+DCALC .EQ. 0.0) THEN
        HT = SSPAR(7) * HOLD
      ELSE 
        HT = (1.0/MAX(LCALC/SSPAR(1), RCALC/SSPAR(2), DCALC/SSPAR(3)))
     &       **(1.0/SSPAR(8)) * HOLD
      ENDIF
C  HT  CONTAINS THE ESTIMATED OPTIMAL STEP SIZE.  NOW PUT IT WITHIN
C REASONABLE BOUNDS.
      H=MIN(MAX(HT,SSPAR(6)*HOLD,SSPAR(4)), SSPAR(7)*HOLD, SSPAR(5))
      IF (ITNUM .EQ. 1) THEN
C IF CONVERGENCE HAD OCCURRED AFTER 1 ITERATION, DON'T DECREASE  H .
        H=MAX(H,HOLD)
      ELSE IF (ITNUM .EQ. LITFH) THEN
C IF CONVERGENCE REQUIRED THE MAXIMUM  LITFH  ITERATIONS, DON'T
C INCREASE  H .
        H=MIN(H,HOLD)
      ENDIF
C IF CONVERGENCE DID NOT OCCUR IN  LITFH  ITERATIONS FOR A PARTICULAR
C H = HFAIL , DON'T CHOOSE THE NEW STEP SIZE LARGER THAN  HFAIL .
      IF (FAIL) H=MIN(H,HFAIL)
C
C
      RETURN
      END

      subroutine fixpnf(n,y,iflag,arcre,arcae,ansre,ansae,trace,a,nfe,
     &   arclen,yp,yold,ypold,qr,alpha,tz,pivot,w,wp,z0,z1,sspar,
     &   par,ipar)
c
c subroutine  fixpnf  finds a fixed point or zero of the
c n-dimensional vector function f(x), or tracks a zero curve
c of a general homotopy map rho(a,lambda,x).  for the fixed 
c point problem f(x) is assumed to be a c2 map of some ball 
c into itself.  the equation  x = f(x)  is solved by
c following the zero curve of the homotopy map
c
c  lambda*(x - f(x)) + (1 - lambda)*(x - a)  ,
c
c starting from lambda = 0, x = a.  the curve is parameterized
c by arc length s, and is followed by solving the ordinary
c differential equation  d(homotopy map)/ds = 0  for
c y(s) = (lambda(s), x(s)) using a hermite cubic predictor and a
c corrector which returns to the zero curve along the flow normal
c to the davidenko flow (which consists of the integral curves of
c d(homotopy map)/ds ).
c
c for the zero finding problem f(x) is assumed to be a c2 map
c such that for some r > 0,  x*f(x) >= 0  whenever norm(x) = r.
c the equation  f(x) = 0  is solved by following the zero curve
c of the homotopy map
c
c   lambda*f(x) + (1 - lambda)*(x - a)
c
c emanating from lambda = 0, x = a.
c
c  a  must be an interior point of the above mentioned balls.
c
c for the curve tracking problem rho(a,lambda,x) is assumed to
c be a c2 map from e**m x [0,1) x e**n into e**n, which for
c almost all parameter vectors a in some nonempty open subset
c of e**m satisfies
c
c  rank [d rho(a,lambda,x)/d lambda , d rho(a,lambda,x)/dx] = n
c
c for all points (lambda,x) such that rho(a,lambda,x)=0.  it is
c further assumed that
c
c           rank [ d rho(a,0,x0)/dx ] = n  .
c
c with a fixed, the zero curve of rho(a,lambda,x) emanating
c from  lambda = 0, x = x0  is tracked until  lambda = 1  by
c solving the ordinary differential equation
c d rho(a,lambda(s),x(s))/ds = 0  for  y(s) = (lambda(s), x(s)),
c where s is arc length along the zero curve.  also the homotopy
c map rho(a,lambda,x) is assumed to be constructed such that
c
c              d lambda(0)/ds > 0  .
c
c
c for the fixed point and zero finding problems, the user must supply 
c a subroutine  f(x,v)  which evaluates f(x) at x and returns the 
c vector f(x) in v, and a subroutine  fjac(x,v,k)  which returns in v 
c the kth column of the jacobian matrix of f(x) evaluated at x.  for 
c the curve tracking problem, the user must supply a subroutine  
c  rho(a,lambda,x,v,par,ipar)  which evaluates the homotopy map rho at 
c (a,lambda,x) and returns the vector rho(a,lambda,x) in v, and a
c subroutine  rhojac(a,lambda,x,v,k,par,ipar)  which returns in v the kth
c column of the n x (n+1) jacobian matrix [d rho/d lambda, d rho/dx]
c evaluated at (a,lambda,x).  fixpnf  directly or indirectly uses
c the subroutines  stepnf , tangnf , rootnf , root , f (or  rho ),
c fjac (or  rhojac ), d1mach , and the blas functions  ddot  and
c dnrm2 .  only  d1mach  contains machine dependent constants.
c no other modifications by the user are required.
c
c
c on input:
c
c n  is the dimension of x, f(x), and rho(a,lambda,x).
c
c y  is an arrray of length  n + 1.  (y(2),...,y(n+1)) = a  is the
c    starting point for the zero curve for the fixed point and 
c    zero finding problems.  (y(2),...,y(n+1)) = x0  for the curve
c    tracking problem.
c
c iflag  can be -2, -1, 0, 2, or 3.  iflag  should be 0 on the 
c    first call to  fixpnf  for the problem  x=f(x), -1 for the
c    problem  f(x)=0, and -2 for the problem  rho(a,lambda,x)=0.
c    in certain situations  iflag  is set to 2 or 3 by  fixpnf,
c    and  fixpnf  can be called again without changing  iflag.
c
c arcre , arcae  are the relative and absolute errors, respectively,
c    allowed the normal flow iteration along the zero curve.  if
c    arc?e .le. 0.0  on input it is reset to  .5*sqrt(ans?e) .
c    normally  arc?e should be considerably larger than  ans?e .
c
c ansre , ansae  are the relative and absolute error values used for
c    the answer at lambda = 1.  the accepted answer  y = (lambda, x)
c    satisfies
c
c       |y(1) - 1|  .le.  ansre + ansae           .and.
c
c       ||z||  .le.  ansre*||x|| + ansae          where
c
c    (.,z) is the newton step to y.
c
c trace  is an integer specifying the logical i/o unit for
c    intermediate output.  if  trace .gt. 0  the points computed on
c    the zero curve are written to i/o unit  trace .
c
c a(1:*)  contains the parameter vector  a .  for the fixed point
c    and zero finding problems, a  need not be initialized by the
c    user, and is assumed to have length  n.  for the curve
c    tracking problem, a  must be initialized by the user.
c
c yp(1:n+1)  is a work array containing the tangent vector to 
c    the zero curve at the current point  y .
c
c yold(1:n+1)  is a work array containing the previous point found
c    on the zero curve.
c
c ypold(1:n+1)  is a work array containing the tangent vector to 
c    the zero curve at  yold .
c
c qr(1:n,1:n+2), alpha(1:n), tz(1:n+1), pivot(1:n+1) , w(1:n+1) ,
c    wp(1:n+1) , z0(1:n+1) , z1(1:n+1)  are all work arrays used by
c    stepnf  to calculate the tangent vectors and newton steps.
c
c sspar(1:8) = (lideal, rideal, dideal, hmin, hmax, bmin, bmax, p)  is
c    a vector of parameters used for the optimal step size estimation.
c    if  sspar(j) .le. 0.0  on input, it is reset to a default value
c    by  fixpnf .  otherwise the input value of  sspar(j)  is used.
c    see the comments below and in  stepnf  for more information about
c    these constants.
c
c par(1:*) and ipar(1:*) are arrays for (optional) user parameters,
c    which are simply passed through to the user written subroutines
c    rho, rhojac.
c
c
c on output:
c
c n , trace , a  are unchanged.
c
c y(1) = lambda, (y(2),...,y(n+1)) = x, and y is an approximate
c    zero of the homotopy map.  normally lambda = 1 and x is a
c    fixed point(zero) of f(x).  in abnormal situations lambda
c    may only be near 1 and x is near a fixed point(zero).
c
c iflag =
c  -2   causes  fixpnf  to initialize everything for the problem
c       rho(a,lambda,x) = 0 (use on first call).
c
c  -1   causes  fixpnf  to initialize everything for the problem
c       f(x) = 0 (use on first call).
c
c   0   causes  fixpnf  to initialize everything for the problem
c       x = f(x) (use on first call).
c
c   1   normal return.
c
c   2   specified error tolerance cannot be met.  some or all of
c       arcre , arcae , ansre , ansae  have been increased to 
c       suitable values.  to continue, just call  fixpnf  again 
c       without changing any parameters.
c
c   3   stepnf  has been called 1000 times.  to continue, call
c       fixpnf  again without changing any parameters.
c
c   4   jacobian matrix does not have full rank.  the algorithm
c       has failed (the zero curve of the homotopy map cannot be
c       followed any further).
c
c   5   the tracking algorithm has lost the zero curve of the
c       homotopy map and is not making progress.  the error tolerances
c       arc?e  and  ans?e  were too lenient.  the problem should be
c       restarted by calling  fixpnf  with smaller error tolerances
c       and  iflag = 0 (-1, -2).
c
c   6   the normal flow newton iteration in  stepnf  or  rootnf
c       failed to converge.  the error tolerances  ans?e  may be too
c       stringent.
c
c   7   illegal input parameters, a fatal error.
c
c arcre , arcae , ansre , ansae  are unchanged after a normal return 
c    (iflag = 1).  they are increased to appropriate values on the 
c    return  iflag = 2 .
c
c nfe  is the number of function evaluations (= number of
c    jacobian evaluations).
c
c arclen  is the length of the path followed.
c
c
c
      double precision abserr,ansae,ansre,arcae,arclen,arcre,
     1   cursw,curtol,d1mach,dnrm2,h,hold,relerr,s
      integer iflag,iflagc,iter,jw,limit,limitd,n,nc,nfe,nfec,np1,
     1   trace
      logical crash,polsys,start
c
c ***** array declarations. *****
c
      double precision y(n+1),yp(n+1),yold(n+1),ypold(n+1),a(n),
     &   qr(n,n+2),alpha(n),tz(n+1),w(n+1),wp(n+1),z0(n+1),
     &   z1(n+1),sspar(8),par(1)
      integer pivot(n+1),ipar(1)
*-----------------------------------------------------------------------
*-----------------------------------------------------------------------
c
c
c ***** end of dimensional information. *****
c
      save
c
c limitd  is an upper bound on the number of steps.  it may be
c changed by changing the following parameter statement:
      parameter (limitd=10000)
c
c switch from the tolerance  arc?e  to the (finer) tolerance  ans?e  if
c the curvature of any component of  y  exceeds  cursw.
      parameter (cursw=10.0)
c
c
c
c :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :  :
c set logical switch to reflect entry point.
      polsys=.false.
      go to 11
*      entry polynf(n,y,iflag,arcre,arcae,ansre,ansae,trace,a,nfe,
*     &   arclen,yp,yold,ypold,qr,alpha,tz,pivot,w,wp,z0,z1,sspar,
*     &   par,ipar)
*      polsys=.true.
11    continue
c
      if (n .le. 0  .or.  ansre .le. 0.0  .or.  ansae .lt. 0.0)
     &                                                     iflag=7
      if (iflag .ge. -2  .and.  iflag .le. 0) go to 20
      if (iflag .eq. 2) go to 120
      if (iflag .eq. 3) go to 90
c only valid input for  iflag  is -2, -1, 0, 2, 3.
      iflag=7
      return
c
c *****  initialization block.  *****
c
20    arclen=0.0
      if (arcre .le. 0.0) arcre=.5*sqrt(ansre)
      if (arcae .le. 0.0) arcae=.5*sqrt(ansae)
      nc=n
      nfec=0
      iflagc=iflag
      np1=n+1
c set initial conditions for first call to  stepnf .
      start=.true.
      crash=.false.
      hold=1.0
      h=.1
      s=0.0
      ypold(1)=1.0
      yp(1)=1.0
      y(1)=0.0
      do 40 jw=2,np1
        ypold(jw)=0.0
        yp(jw)=0.0
40    continue
c set optimal step size estimation parameters.
c let z[k] denote the newton iterates along the flow normal to the
c davidenko flow and y their limit.
c ideal contraction factor:  ||z[2] - z[1]|| / ||z[1] - z[0]||
      if (sspar(1) .le. 0.0) sspar(1)= .5
c ideal residual factor:  ||rho(a, z[1])|| / ||rho(a, z[0])||
      if (sspar(2) .le. 0.0) sspar(2)= .01
c ideal distance factor:  ||z[1] - y|| / ||z[0] - y||
      if (sspar(3) .le. 0.0) sspar(3)= .5
c minimum step size  hmin .
      if (sspar(4) .le. 0.0) sspar(4)= (sqrt(n+1.0)+4.0)*d1mach(4)
c maximum step size  hmax .
      if (sspar(5) .le. 0.0) sspar(5)= 1.00
c minimum step size reduction factor  bmin .
      if (sspar(6) .le. 0.0) sspar(6)= .1
c maximum step size expansion factor  bmax .
      if (sspar(7) .le. 0.0) sspar(7)= 3.0
c assumed operating order  p .
      if (sspar(8) .le. 0.0) sspar(8)= 2.0
c
c load  a  for the fixed point and zero finding problems.
      if (iflagc .ge. -1) then
        do 60 jw=2,np1
          a(jw-1)=y(jw)
60      continue
      endif
90    limit=limitd
c
c *****  end of initialization block.  *****
c
c
c *****  main loop.  *****
c
120   do 400 iter=1,limit
      if (y(1) .lt. 0.0) then
        arclen=s
        iflag=5
        return
      endif
c
c set different error tolerance if the trajectory y(s) has any high 
c curvature components.
140   curtol=cursw*hold
      relerr=arcre
      abserr=arcae
      do 160 jw=1,np1
        if (abs(yp(jw)-ypold(jw)) .gt. curtol) then
          relerr=ansre
          abserr=ansae
          go to 200
        endif
160   continue
c
c take a step along the curve.
200   call stepnf(nc,nfec,iflagc,start,crash,hold,h,relerr,abserr,
     +     s,y,yp,yold,ypold,a,qr,alpha,tz,pivot,w,wp,z0,z1,sspar,
     +     par,ipar)
c print latest point on curve if requested.
      if (trace .gt. 0) then
c        write (trace,217) iter,nfec,s,y(1),(y(jw),jw=2,np1)
c        write (6,217) iter,nfec,s,y(1),(y(jw),jw=2,np1)
217     format(/' step',i5,3x,'nfe =',i5,3x,'arc length =',f9.4,3x,
     &  'lambda =',f7.4,5x,'x vector:'/1p,(1x,6e12.4))
      endif
      nfe=nfec
*-----------------------------------------------------------------------
c     print*,iter,nfec,s,y(1)
      write(6,221)iter,nfec,s,y(1)
c     write(2,221)iter,nfec,s,y(1)
 221   format('step=',i4,' nfe=',i5,' arc=',f9.4,' lambda=',f19.16)
      if(y(1).gt.0.99) then
      open(3 , file='var.dat', status = 'unknown')
      write(3,222)y(1), y(2), y(4), y(8),y(16)
      write(6,222)y(1), y(2), y(4), y(8),y(16)
 222   format(e12.5,x,e12.5, x, e12.5,x,e12.5,x,e12.5)
      close(3)
      endif
c     z = ld * ld
c     count = count + 1
c     call fls(ld,nn,nm,count,par(4*z+1),par(5*z+1),par(6*z+1),y)
*
c     goto 401
c       call symmet(ld,nn,nm,y(2),par(7*z+1),par(8*z+1),par(9*z+1))
c       call ccurv2(ld,nn,nm,y,par,ipar,par(4*z+1),par(5*z+1),
c    &              par(6*z+1),par(10*z+1),par(11*z+1),par(12*z+1),
c    &              par(13*z+1),par(15*z+1))
c401   continue
*-----------------------------------------------------------------------
c check if the step was successful.
      if (iflagc .gt. 0) then
        arclen=s
        iflag=iflagc
        return
      endif
      if (crash) then
c return code for error tolerance too small.
        iflag=2
c change error tolerances.
        if (arcre .lt. relerr) arcre=relerr
        if (ansre .lt. relerr) ansre=relerr
        if (arcae .lt. abserr) arcae=abserr
        if (ansae .lt. abserr) ansae=abserr
c change limit on number of iterations.
        limit=limit-iter
        return
      endif
c
      if (y(1) .ge. 1.0) then
c
c use hermite cubic interpolation and newton iteration to get the 
c answer at lambda = 1.0 .
c
c save  yold  for arc length calculation later.
        do 260 jw=1,np1
          z0(jw)=yold(jw)
260     continue
        call rootnf(nc,nfec,iflagc,ansre,ansae,y,yp,yold,ypold,
     &              a,qr,alpha,tz,pivot,w,wp,par,ipar)
c
        nfe=nfec
        iflag=1
c set error flag if  rootnf  could not get the point on the zero
c curve at  lambda = 1.0  .
        if (iflagc .gt. 0) iflag=iflagc
c calculate final arc length.
        do 290 jw=1,np1
          w(jw)=y(jw) - z0(jw)
290     continue
*-----------------------------------------------------------------------
c       open(14,file='bg.end')
c       write(14,261)(y(jw),jw=1,np1)
c261     format(f19.16)
c       close(14)
*-----------------------------------------------------------------------
        arclen=s - hold + dnrm2(np1,w,1)
        return
      endif
c
c for polynomial systems and the  polsys  homotopy map,
c d lambda/ds .ge. 0 necessarily.  this condition is forced here if
c the entry point was  polynf .
c
      if (polsys) then
        if (yp(1) .lt. 0.0) then
c reverse tangent direction so d lambda/ds = yp(1) > 0 .
          do 310 jw=1,np1
            yp(jw)=-yp(jw)
            ypold(jw)=yp(jw)
310       continue
c force  stepnf  to use the linear predictor for the next step only.
          start=.true.
        endif
      endif
c
400   continue
c
c *****  end of main loop.  *****
c
c lambda has not reached 1 in 1000 steps.
      iflag=3
      arclen=s
      return
c
      end
c
c This subroutine is modified from TANGNF from HOMPACK
c
      subroutine tangnf(rholen,y,yp,ypold,a,qr,alpha,tz,pivot,
     &                  nfe,n,iflag,par,ipar)
c
c this subroutine builds the jacobian matrix of the homotopy map,
c computes a qr decomposition of that matrix, and then calculates the
c (unit) tangent vector and the newton step.
c
c on input:
c
c rholen < 0 if the norm of the homotopy map evaluated at
c    (a, lambda, x) is to be computed.  if  rholen >= 0  the norm is not
c    computed and  rholen  is not changed.
c
c y(1:n+1) = current point (lambda(s), x(s)).
c
c ypold(1:n+1) = unit tangent vector at previous point on the zero
c    curve of the homotopy map.
c
c a(1:*) = parameter vector in the homotopy map.
c
c qr(1:n,1:n+2), alpha(1:n), tz(1:n+1), pivot(1:n+1)  are work arrays
c    used for the qr factorization.
c
c nfe = number of jacobian matrix evaluations = number of homotopy
c    function evaluations.
c
c n = dimension of x.
c
c iflag = -2, -1, or 0, indicating the problem type.
c
c par(1:*) and ipar(1:*) are arrays for (optional) user parameters,
c    which are simply passed through to the user written subroutines
c    rho, rhojac.
c
c on output:
c
c rholen = ||rho(a, lambda(s), x(s)|| if  rholen < 0  on input.
c    otherwise  rholen  is unchanged.
c
c y, ypold, a, n  are unchanged.
c
c yp(1:n+1) = dy/ds = unit tangent vector to integral curve of
c    d(homotopy map)/ds = 0  at  y(s) = (lambda(s), x(s)) .
c
c tz = the newton step = -(pseudo inverse of  (d rho(a,y(s))/d lambda ,
c    d rho(a,y(s))/dx)) * rho(a,y(s)) .
c
c nfe  has been incrmented by 1.
c
c iflag  is unchanged, unless the qr factorization detects a rank < n,
c    in which case the tangent and newton step vectors are not computed
c    and  tangnf  returns with  iflag = 4 .
c
c
c calls  ddot , dnrm2 , f (or  rho ), fjac (or  rhojac ).
c
      double precision ddot,dnrm2,lambda,rholen,
     &       sigma,sum,ypnorm
      integer i,iflag,j,k,n,nfe,np1,np2
c
c *****  array declarations.  *****
c
      double precision y(n+1),yp(n+1),ypold(n+1),a(n),par(10)
      integer ipar(1)
c
c arrays for computing the jacobian matrix and its kernel.
      double precision qr(n,n+2),alpha(n),tz(n+1)
      integer pivot(n+1)
c************************************************************************
c arrays for computing the jacobian matrix's singular value & cond.number
c
c     double precision sv1(20),temp1(20,22),temp2(20,22)
c     integer ipa(20)
c***********************************************************************    
c
c *****  end of dimensional information.  *****
c
c
      lambda=y(1)
      np1=n+1
      np2=n+2
      nfe=nfe+1
c nfe contains the number of jacobian evaluations.
c   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *
c
c compute the jacobian matrix, store it and homotopy map in qr.
c
      if (iflag .eq. -2) then
c
c  qr = ( d rho(a,lambda,x)/d lambda , d rho(a,lambda,x)/dx ,
c                                              rho(a,lambda,x) )  .
c
        call rho(a,lambda,y(2),qr(1,np2),par,ipar)
        do 30 k=1,np1
          call rhojac(a,lambda,y(2),qr(1,k),k,par,ipar)
30      continue
********************************************************
c       ipar(1) = ipar(1) + 1
c       if(ipar(1) .eq. 6) then
c       open(3,file='mat.dat')
c	do 313 i=1,n
c	  do 313 j=1,np1
c	  if(qr(i,j).ne.0.0d0) then
c         write(3,211)i,j,qr(i,j)
c211       format(i3,2x,i3,2x,e25.15)
c         endif
c313     continue
c       close(3)
c       stop
c	endif
c         do  356 i=1,n
c           do  356 j=1,n+1
c      	    temp1(i,j) = qr(i,j)
c356      continue 
c         call dsvdc(temp1,20,n,n+1,sv1,par(1),par(n+1),
c    &          n,par(n*n+n+1),n+1,par(3+5*n+2*n*n),00,info)
c	write(6,213)sv1(1)/sv1(n),info
c213     format(/,'Cond Number',2x,d9.2,2x,i3)
********************************************************
        else
c      call f(y(2),tz)
c      if (iflag .eq. 0) then
c
c      qr = ( a - f(x), i - lambda*df(x) ,
c                                 x - a + lambda*(a - f(x)) )  .
c
c         do 100 j=1,n
c           sigma=a(j)
c           beta=sigma-tz(j)
c           qr(j,1)=beta
c100       qr(j,np2)=y(j+1)-sigma+lambda*beta
c         do 120 k=1,n
c           call fjac(y(2),tz,k)
c           kp1=k+1
c           do 110 j=1,n
c110         qr(j,kp1)=-lambda*tz(j)
c120       qr(k,kp1)=1.0+qr(k,kp1)
c       else
c
c   qr = ( f(x) - x + a, lambda*df(x) + (1 - lambda)*i ,
c                                  x - a + lambda*(f(x) - x + a) )  .
c
c140       do 150 j=1,n
c           sigma=y(j+1)-a(j)
c           beta=tz(j)-sigma
c           qr(j,1)=beta
c150       qr(j,np2)=sigma+lambda*beta
c         do 170 k=1,n
c           call fjac(y(2),tz,k)
c           kp1=k+1
c           do 160 j=1,n
c160         qr(j,kp1)=lambda*tz(j)
c170       qr(k,kp1)=1.0-lambda+qr(k,kp1)
c       endif
      endif
c
c   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *
c compute the norm of the homotopy map if it was requested.
      if (rholen .lt. 0.0) rholen=dnrm2(n,qr(1,np2),1)
c
c reduce the jacobian matrix to upper triangular form.
c
c the following code is a modification of the algol procedure
c decompose  in p. businger and g. h. golub, linear least
c squares solutions by householder transformations,
c numer. math. 7 (1965) 269-276.
c
      ind = 1
      do 344 i=1,n+1
        pivot(i) = 0
344   continue
        pivot(n+2) =-1
      call dqrdc(qr,n,n,n+2,par(1),pivot,par(n+2),ind)
c     write(2,203) (qr(i,i),i=1,n)
c203   format(/,(d25.15))
c      write(2,204) (ipa(i),i=1,n)
c204   format(/,(i3))
      do 345 i=1,n
       alpha(i) = qr(i,i)
345   continue
c
c
c compute kernel of jacobian, which specifies yp=dy/ds.
      tz(np1)=1.0
      do 340 i=n,1,-1
        sum=0.0
        do 330 j=i+1,np1
330     sum=sum+qr(i,j)*tz(j)
340   tz(i)=-sum/alpha(i)
      ypnorm=dnrm2(np1,tz,1)
      do 360 k=1,np1
360   yp(pivot(k))=tz(k)/ypnorm
      if (ddot(np1,yp,1,ypold,1) .ge. 0.0) go to 380
      do 370 i=1,np1
370   yp(i)=-yp(i)
c yp  is the unit tangent vector in the correct direction.
c
c compute the minimum norm solution of [d rho(y(s))] v = -rho(y(s)).
c v is given by  p - (p,q)q  , where p is any solution of
c [d rho] v = -rho  and q is a unit vector in the kernel of [d rho].
c
380   do 440 i=n,1,-1
        sum=qr(i,np1)+qr(i,np2)
        do 430 j=i+1,n
430     sum=sum+qr(i,j)*alpha(j)
440   alpha(i)=-sum/alpha(i)
      do 450 k=1,n
450   tz(pivot(k))=alpha(k)
      tz(pivot(np1))=1.0
c tz now contains a particular solution p, and yp contains a vector q
c in the kernel(the tangent).
      sigma=ddot(np1,tz,1,yp,1)
      do 470 j=1,np1
        tz(j)=tz(j)-sigma*yp(j)
470   continue
c tz is the newton step from the current point y(s) = (lambda(s), x(s)).
      return
      end
