#include <stdio.h>

void nrerror(error_text)
char error_text[];
{
	void exit();

	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}



float *vector(nl,nh)
int nl,nh;
{
	float *v;

	v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl;
}

int *ivector(nl,nh)
int nl,nh;
{
	int *v;

	v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
	if (!v) nrerror("allocation failure in ivector()");
	return v-nl;
}

double *dvector(nl,nh)
int nl,nh;
{
	double *v;

	v=(double *)malloc((unsigned) (nh-nl+1)*sizeof(double));
	if (!v) nrerror("allocation failure in dvector()");
	return v-nl;
}



float **matrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
	int i;
	float **mp;

	mp=(float **) malloc((unsigned) (nrh-nrl+1)*sizeof(float*));
	if (!mp) nrerror("allocation failure 1 in matrix()");
	mp -= nrl;

	for(i=nrl;i<=nrh;i++) {
		mp[i]=(float *) malloc((unsigned) (nch-ncl+1)*sizeof(float));
		if (!mp[i]) nrerror("allocation failure 2 in matrix()");
		mp[i] -= ncl;
	}
	return mp;
}

double **dmatrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
	int i;
	double **mp;

	mp=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
	if (!mp) nrerror("allocation failure 1 in dmatrix()");
	mp -= nrl;

	for(i=nrl;i<=nrh;i++) {
		mp[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
		if (!mp[i]) nrerror("allocation failure 2 in dmatrix()");
		mp[i] -= ncl;
	}
	return mp;
}

int **imatrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
	int i,**mp;

	mp=(int **)malloc((unsigned) (nrh-nrl+1)*sizeof(int*));
	if (!mp) nrerror("allocation failure 1 in imatrix()");
	mp -= nrl;

	for(i=nrl;i<=nrh;i++) {
		mp[i]=(int *)malloc((unsigned) (nch-ncl+1)*sizeof(int));
		if (!mp[i]) nrerror("allocation failure 2 in imatrix()");
		mp[i] -= ncl;
	}
	return mp;
}



float **submatrix(a,oldrl,oldrh,oldcl,oldch,newrl,newcl)
float **a;
int oldrl,oldrh,oldcl,oldch,newrl,newcl;
{
	int i,j;
	float **mp;

	mp=(float **) malloc((unsigned) (oldrh-oldrl+1)*sizeof(float*));
	if (!mp) nrerror("allocation failure in submatrix()");
	mp -= newrl;

	for(i=oldrl,j=newrl;i<=oldrh;i++,j++) mp[j]=a[i]+oldcl-newcl;

	return mp;
}



void free_vector(v,nl,nh)
float *v;
int nl,nh;
{
	free((char*) (v+nl));
}

void free_ivector(v,nl,nh)
int *v,nl,nh;
{
	free((char*) (v+nl));
}

void free_dvector(v,nl,nh)
double *v;
int nl,nh;
{
	free((char*) (v+nl));
}



void free_matrix(mp,nrl,nrh,ncl,nch)
float **mp;
int nrl,nrh,ncl,nch;
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (mp[i]+ncl));
	free((char*) (mp+nrl));
}

void free_dmatrix(mp,nrl,nrh,ncl,nch)
double **mp;
int nrl,nrh,ncl,nch;
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (mp[i]+ncl));
	free((char*) (mp+nrl));
}

void free_imatrix(mp,nrl,nrh,ncl,nch)
int **mp;
int nrl,nrh,ncl,nch;
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (mp[i]+ncl));
	free((char*) (mp+nrl));
}



void free_submatrix(b,nrl,nrh,ncl,nch)
float **b;
int nrl,nrh,ncl,nch;
{
	free((char*) (b+nrl));
}



float **convert_matrix(a,nrl,nrh,ncl,nch)
float *a;
int nrl,nrh,ncl,nch;
{
	int i,j,nrow,ncol;
	float **mp;

	nrow=nrh-nrl+1;
	ncol=nch-ncl+1;
	mp = (float **) malloc((unsigned) (nrow)*sizeof(float*));
	if (!mp) nrerror("allocation failure in convert_matrix()");
	mp -= nrl;
	for(i=0,j=nrl;i<=nrow-1;i++,j++) mp[j]=a+ncol*i-ncl;
	return mp;
}



void free_convert_matrix(b,nrl,nrh,ncl,nch)
float **b;
int nrl,nrh,ncl,nch;
{
	free((char*) (b+nrl));
}
