function [Ac,Bc,Cc,omegab]=inormric(Ac,Bc,Cc)
%[Ac,Bc,Cc,omegab]=inormric(Ac,Bc,Cc)
%INORMRIC Transforms {Ac,Bc,Cc} to the input normalized basis.

%       910830  A. Daubendiek   created
%       910916  L. Davis     	tested on example 1
%
%       step 1.  Find sizes of objects and allocate space for omegab.
%
[ nc, nu ] = size( Bc ) ;
[ ny, nc ] = size( Cc ) ;
omegab = zeros(nc,nc);
%
%	step 2.  Calculate Riccati solutions Q and P.
%
v1 = Bc*Bc' ;
r1 = Cc'*Cc ;
Inu = eye( nu ) ;
Iny = eye( ny ) ;
[ K, P ] = lqr( Ac, Bc, r1, Inu ) ;
[ K, Q ] = lqr( Ac', Cc', v1, Iny ) ;
%
%       step 3.  Find Cholesky factors of Q and P.
%
Po = P;
Qo = Q;
Lp = real((chol( P ))') ;
Lq = real((chol( Q ))') ;
%
%       step 4.  Find SVD of factors.  Compute sqrt of omegab.
%
[ U, D, V ] = svd( Lp'*Lq ) ;
omegab = diag(D) * ones(1,nc);
%
%       step 5.  Find transformation to input normal form.
%
T = Lq * V;
invT = (U' * Lp')./omegab;
%
%       step 6.  Transform to input normal basis.
%
Ac = invT*Ac*T ;
Bc = invT*Bc ;
Cc = Cc*T ;
omegab = omegab.^2;

ome(1:nc,1:nc) = zeros(nc,nc);
for i=1:nc
   ome(i,i) = omegab(i,i);
end
%ome
%UU = Ac + Ac' + Bc*Bc' -Cc'*Cc
%WW = Ac' * ome+ ome *Ac + Cc'*Cc - ome*Bc*Bc'*ome
%
end

