      subroutine initp4(ld,n,nm,m,l,nn,c1,c2,d,bvbt,ctrc,y,hom,q,p,tau,
     &                  f10,f20,f30,ww,wi,sigma,temp1,temp2,temp3,
     &                  qr,wa,fvec,s1)
*
* Strategy 4 for choosing the starting point for homotopy
*
* Input parameters:
* ld - leading dimension of matrices
* n,nm,m,l - dimensions of system and model
* nn - Jacobian size
* c1,c2 - parameters used in definition of D
* d,bvbt,ctrc - D, B V B^t, C^t R C
*
* Output parameters:
* y - point in vector form
* hom - parameter indicating which homotopy map will be used, depends
*       on the result of quasi-Newton iteration
*
* Working storage:
* q,p,tau,f10,f20,f30,ww,wi,sigma,temp1,temp2,temp3,qr - working matrices
* wa,fvec,s1 - working arrays
*
      integer hom,i,l,ld,m,n,nm,nn,realnm,z
      double precision bvbt(ld,ld),c1,c2,ctrc(ld,ld),d(ld,ld),f10(ld,ld)
      double precision f20(ld,ld),f30(ld,ld),fvec(nn),p(ld,ld),q(ld,ld)
      double precision qr(nn,nn),s1(ld),sigma(ld,ld),small,t1,tau(ld,ld)
      double precision temp1(ld,ld),temp2(ld,ld),temp3(ld,ld)
      double precision wa(nn*(nn+13)/2),wi(ld,ld),ww(ld,ld),y(nn)
*
      z = ld * ld
      small = 1.d-4
*
* Compute solutions of initial problem with ranks m and l
*
      t1 = 1.d0 / (2.d0 * c1)
      call scmat(ld,t1,bvbt,q,n,n)
      call scmat(ld,t1,ctrc,p,n,n)
*
* nm = min(m,l)
*
      realnm = nm
      nm = m
      if (l .lt. nm) nm = l
*
* Compute (W,U,Sigma) for given (Q,P), rank nm-1
*
      call cg(ld,n,nm,q,p,ww,wi,sigma,tau,temp1,temp2,temp3,
     &        wa(1),wa(z+1),wa(2*z+1),wa(3*z+1),wa(4*z+1),wa(5*z+1),
     &        wa(6*z+1),wa(7*z+1),wa(8*z+1),wa(9*z+1),wa(10*z+1),
     &        wa(11*z+1))
*
* Augment Sigma
*
      if (nm .lt. realnm) then
        do 302 i = nm+1,realnm
          sigma(i,i) = small
302     continue
      endif
*
      nm = realnm
*
c     write(2,231)
c231   format(/'sigma')
c     call writem(ld,sigma,nm,nm)
*
* Quasi-Newton iteration
*
*      call apprd(ld,n,nm,nn,d,bvbt,ctrc,y,hom,ww,wi,sigma,f10,f20,f30,
*     &           temp1,temp2,temp3,qr,wa,fvec,s1)
*      if (hom .ne. 1) hom = 0
*
      return
      end
