*
*     MULMAT
*
      subroutine mulmat(ld,a,b,c,m,n,k,type)
*
* Perform matrix-matrix multiplication: C(m,n) = A(m,k) * B(k,n)
* NOTE: A or B may be used as A^t or B^t, depending on type, i.e.,
*       if type=2 then C(m,n) = A^t(m,k) * B(k,n)
* type - 1 - C = A * B
*        2 - C = A^t * B
*        3 - C = A * B^t
*        4 - C = A^t * B^t
*
      integer i,j,l,m,n,k,type,ld
      double precision a(ld,ld),b(ld,ld),c(ld,ld)
*
      goto (401,411,421,431) type
*
401   continue
      do 301 i = 1,m
        do 302 j = 1,n
          c(i,j) = 0.d0
          do 303 l = 1,k
            c(i,j) = c(i,j) + a(i,l) * b(l,j)
303       continue
302     continue
301   continue
      goto 441
*
411   continue
      do 311 i = 1,m
        do 312 j = 1,n
          c(i,j) = 0.d0
          do 313 l = 1,k
            c(i,j) = c(i,j) + a(l,i) * b(l,j)
313       continue
312     continue
311   continue
      goto 441
*      
421   continue
      do 321 i = 1,m
        do 322 j = 1,n
          c(i,j) = 0.d0
          do 323 l = 1,k
            c(i,j) = c(i,j) + a(i,l) * b(j,l)
323       continue
322     continue
321   continue
      goto 441
*      
431   continue
      do 331 i = 1,m
        do 332 j = 1,n
          c(i,j) = 0.d0
          do 333 l = 1,k
            c(i,j) = c(i,j) + a(l,i) * b(j,l)
333       continue
332     continue
331   continue
*
441   continue
*
      return
      end
*
*-------------------------------------------------------------------
*-------------------------------------------------------------------
*
*     ADDMAT
*
      subroutine addmat(ld,a,b,c,m,n,type)
*
* Perform matrix-matrix addition: C(m,n) = A(m,n) + B(m,n)
* NOTE: A or B may be used as A^t or B^t, depending on type, i.e.,
*       if type=2 then C(m,n) = A^t(n,m) + B(m,n)
* type - 1 - C = A * B
*        2 - C = A^t * B
*        3 - C = A * B^t
*        4 - C = A^t * B^t
*
      integer i,j,m,n,type,ld
      double precision a(ld,ld),b(ld,ld),c(ld,ld)
*
      goto (401,411,421,431) type
*
401   continue
      do 301 i = 1,m
        do 302 j = 1,n
          c(i,j) = a(i,j) +  b(i,j)
302     continue
301   continue
      goto 441
*
411   continue
      do 311 i = 1,m
        do 312 j = 1,n
          c(i,j) = -a(i,j) +  b(i,j)
312     continue
311   continue
      goto 441
*
421   continue
      do 321 i = 1,m
        do 322 j = 1,n
          c(i,j) = a(i,j) -  b(i,j)
322     continue
321   continue
      goto 441
*
431   continue
      do 331 i = 1,m
        do 332 j = 1,n
          c(i,j) = -a(i,j) -  b(i,j)
332     continue
331   continue
*
441   continue
*
      return
      end
*
*-------------------------------------------------------------------
*-------------------------------------------------------------------
*
*     SCMAT
*
      subroutine scmat(ld,s,a,b,m,n)
*
* Scalar multiplication of matrix: B(m,n) = s * A(m,n)
*
      integer i,j,m,n,ld
      double precision a(ld,ld),b(ld,ld)
      double precision s
*
      do 301 i = 1,m
        do 302 j = 1,n
          b(i,j) = s * a(i,j)
302     continue
301   continue
*
      return
      end
*
*-------------------------------------------------------------------
*-------------------------------------------------------------------
*
*     SETMAT
*
      subroutine setmat(ld,s,a,m,n)
*
* Set all elements of A(m,n) to scalar s
*
      integer i,j,m,n,ld
      double precision a(ld,ld)
      double precision s
*
      do 301 i = 1,m
        do 302 j = 1,n
          a(i,j) = s
302     continue
301   continue
*
      return
      end
*
*-------------------------------------------------------------------
*-------------------------------------------------------------------
*
*     TRANS
*
      subroutine trans(ld,a,b,m,n)
*
* B(n,m) = A(m,n)^t
*
      integer i,j,m,n,ld
      double precision a(ld,ld),b(ld,ld)
*
      do 301 i = 1,n
        do 302 j = 1,m
          b(i,j) = a(j,i)
302     continue
301   continue
*
      return
      end
*
*-------------------------------------------------------------------
*-------------------------------------------------------------------
*
*     ROWCOL
*
      subroutine rowcol(ld,mat1,mat2,length,fix1,fix2,type)
*
* Depending on type set certain row (column) of MAT1 to row (column) of MAT2
* type - 1 - fix1 row of MAT1 = fix2 row of MAT2
*        2 - fix1 row of MAT1 = fix2 column of MAT2
*        3 - fix1 column of MAT1 = fix2 row of MAT2
*        4 - fix1 column of MAT1 = fix2 column of MAT2
*
      integer i,length,fix1,fix2,type,ld
      double precision mat1(ld,ld),mat2(ld,ld)
*
      goto (401,411,421,431) type
*
401   continue
      do 301 i = 1,length
        mat1(fix1,i) = mat1(fix1,i) + mat2(fix2,i)
301   continue
      goto 441
*
411   continue
      do 311 i = 1,length
        mat1(fix1,i) = mat1(fix1,i) + mat2(i,fix2)
311   continue
      goto 441
*
421   continue
      do 321 i = 1,length
        mat1(i,fix1) = mat1(i,fix1) + mat2(fix2,i)
321   continue
      goto 441
*
431   continue
      do 331 i = 1,length
        mat1(i,fix1) = mat1(i,fix1) + mat2(i,fix2)
331   continue
*
441   continue
*
      return
      end
*
*-------------------------------------------------------------------
*-------------------------------------------------------------------
*
*     SETVEC
*
      subroutine setvec(ld,s,a,n)
*
* Set all elements of A(n) to scalar s
*
      integer i,n,ld
      double precision a(ld)
      double precision s
*
      do 301 i = 1,n
        a(i) = s
301   continue
*
      return
      end
*
*-------------------------------------------------------------------
*-------------------------------------------------------------------
*
*     WRITEM
*
      subroutine writem(ld,mat,row,col)
*
      integer ld,row,col,i,j
      double precision mat(ld,ld)
*
      write(2,200)
200   format(' ')
      do 301 i = 1,row
        do 302 j = 1,col
          write(2,201)i,j,mat(i,j)
201       format(2i3,2f27.16)
302     continue
301   continue
*
      return
      end
*
*-------------------------------------------------------------------
*-------------------------------------------------------------------
*
*     WRITMD
*
      subroutine writmd(ld,mat,row,col)
*
      integer ld,row,col,i,j
      double precision mat(ld,ld)
*
      write(2,200)
200   format(' ')
      do 301 i = 1,row
        do 302 j = 1,col
          write(2,201)i,j,mat(i,j)
201       format(2i3,1p,(2d25.16))
302     continue
301   continue
*
      return
      end
*
*-------------------------------------------------------------------
*-------------------------------------------------------------------
*
*     WRITEV
*
      subroutine writev(vec,dim)
*
      double precision vec(*)
      integer dim,i
*
      write(2,200)
200   format(' ')
      do 301 i = 1,dim
        write(2,201)i,vec(i)
201     format(i3,f27.16)
301   continue
*
      return
      end
*
*-------------------------------------------------------------------
*-------------------------------------------------------------------
*
*     WRITVD
*
      subroutine writvd(vec,dim)
*
      double precision vec(*)
      integer dim,i
*
      write(2,200)
200   format(' ')
      do 301 i = 1,dim
        write(2,201)i,vec(i)
201     format(i3,1p,(d25.16))
301   continue
*
      return
      end
*
