      subroutine rrhoj(ld,n,nm,type,hom,lambda,x,kk,a,d,bvbt,ctrc,
     &                 f10,f20,f30,v,ww,wi,sigma,temp1,temp2,temp3,aal,
     &                 mat,amd)
*
* RRHOJ evaluates kk-th column of Jacobian at given point.
*
* Input parameters:
* ld - leading dimension of matrices
* n,nm - dimensions of system and model
* type - indicates whether RRHO is used for curve tracking (type = 1)
*        or quasi-Newton iterations (type = 2 or 3)
* hom - indicates type of homotopy map: hom = 0 indicates Newton
*       homotopy, hom = 1 indicates rho(a,lambda,x) = F(a,lambda,x)
* lambda,x - input point (lambda,x)
* kk - column number
* a,d,bvbt,ctrc - A,D, B V B^t, C^t R C
* f10,f20,f30 - F(0,X(0))
*
* Output parameters:
* v - evaluated function
*
* Working storage:
* ww,wi,sigma,temp1,temp2,temp3,aal,mat,amd - working matrices
*
      integer der,hom,i,j,k,kk,ld,n,nm,type
      double precision a(ld,ld),aal(ld,ld),amd(20,20),bvbt(ld,ld)
      double precision ctrc(ld,ld),d(ld,ld),f10(ld,ld),f20(ld,ld)
      double precision f30(ld,ld),lambda,mat(ld,ld),sigma(ld,ld),t1
      double precision temp1(ld,ld),temp2(ld,ld),temp3(ld,ld),v(*)
      double precision wi(ld,ld),ww(ld,ld)
*
      call xwus(ld,n,nm,1,ww,wi,sigma,x)
*
* Compute AAL = lambda * A + (1-lambda) * D
*
      t1 = 1.d0 - lambda
      call scmat(ld,lambda,a,temp1,n,n)
      call scmat(ld,t1,d,temp2,n,n)
      call addmat(ld,temp1,temp2,aal,n,n,1)
*
* Determine which matrix is used as A(lambda)
*
      if (type .eq. 1) call scmat(ld,1.d0,aal,mat,n,n)
      if (type .eq. 2) call scmat(ld,1.d0,d,mat,n,n)
      if (type .eq. 3) call scmat(ld,1.d0,a,mat,n,n)
*
* Determine variable with respect to which the derivative is sought
* der=4 - lambda, der=1 - W, der=2 - U, der=3 - Sigma
*
      if (type .eq. 1) k = kk
      if (type .eq. 2) k = kk + 1
      if (type .eq. 3) k = kk + 1
*
      if (k .eq. 1) der = 4
      if (k .gt. 1 .and. k .le. n*nm+1) der = 1
      if (k .gt. n*nm+1 .and. k .le. 2*n*nm+1) der = 2
      if (k .gt. 2*n*nm+1) der = 3
*
* Determine position (i,j) in W, U or Sigma
*
      if (der .eq. 1) call compij(k-1,n,nm,i,j)
      if (der .eq. 2) call compij(k-n*nm-1,nm,n,i,j)
      if (der .eq. 3) call compij(k-2*n*nm-1,nm,nm,i,j)
*.......................................................................
*
* Derivatives with respect to Wij
*
      if (der .eq. 1) then
*
* Compute derivatives of F1(a,lambda,x)
*
        call mulmat(ld,wi,mat,temp1,nm,n,n,1)
        call mulmat(ld,temp1,ww,temp2,nm,nm,n,1)
        call mulmat(ld,temp2,sigma,temp1,nm,nm,nm,1)
        call setmat(ld,0.d0,temp3,nm,n)
        call rowcol(ld,temp3,temp1,nm,i,j,4)
*
        call mulmat(ld,wi,mat,temp1,nm,n,n,1)
        call setmat(ld,0.d0,temp2,nm,nm)
        call rowcol(ld,temp2,temp1,nm,j,i,4)
        call mulmat(ld,temp2,sigma,temp1,nm,nm,nm,1)
        call mulmat(ld,temp1,ww,temp2,nm,n,nm,3)
        call addmat(ld,temp2,temp3,temp3,nm,n,1)
*
        call setmat(ld,0.d0,temp1,nm,n)
        call rowcol(ld,temp1,sigma,nm,i,j,4)
        call mulmat(ld,temp1,mat,temp2,nm,n,n,3)
        call addmat(ld,temp2,temp3,temp3,nm,n,1)
        call xmat(ld,2,nm,n,temp3,v)
*
* Compute derivatives of F2(a,lambda,x)
*
        call mulmat(ld,wi,sigma,temp1,n,nm,nm,2)
        call mulmat(ld,temp1,wi,temp2,n,n,nm,1)
        call mulmat(ld,temp2,mat,temp1,n,n,n,1)
        call setmat(ld,0.d0,temp3,n,nm)
        call rowcol(ld,temp3,temp1,n,j,i,4)
        call rowcol(ld,temp3,ctrc,n,j,i,4)
        call xmat(ld,2,n,nm,temp3,v(1+n*nm))
*
* Compute derivatives of F3(a,lambda,x)
*
        call setmat(ld,0.d0,temp3,nm,nm)
        call rowcol(ld,temp3,wi,nm,j,i,4)
        call xmat(ld,2,nm,nm,temp3,v(1+2*n*nm))
      endif
*.......................................................................
*
* Derivatives with respect to Uij
*
      if (der .eq. 2) then
*
* Compute derivatives of F1(a,lambda,x)
*
        call mulmat(ld,mat,ww,temp1,n,nm,n,1)
        call mulmat(ld,temp1,sigma,temp2,n,nm,nm,1)
        call mulmat(ld,temp2,ww,temp1,n,n,nm,3)
        call setmat(ld,0.d0,temp3,nm,n)
        call rowcol(ld,temp3,temp1,n,i,j,1)
        call rowcol(ld,temp3,bvbt,n,i,j,1)
        call xmat(ld,2,nm,n,temp3,v)
*
* Compute derivatives of F2(a,lambda,x)
*
        call mulmat(ld,sigma,wi,temp1,nm,n,nm,1)
        call mulmat(ld,temp1,mat,temp2,nm,n,n,1)
        call mulmat(ld,temp2,ww,temp1,nm,nm,n,1)
        call setmat(ld,0.d0,temp3,n,nm)
        call rowcol(ld,temp3,temp1,nm,j,i,1)
*
        call mulmat(ld,wi,sigma,temp1,n,nm,nm,2)
        call setmat(ld,0.d0,temp2,n,n)
        call rowcol(ld,temp2,temp1,n,j,i,4)
        call mulmat(ld,temp2,mat,temp1,n,n,n,1)
        call mulmat(ld,temp1,ww,temp2,n,nm,n,1)
        call addmat(ld,temp2,temp3,temp3,n,nm,1)
*
        call setmat(ld,0.d0,temp1,n,nm)
        call rowcol(ld,temp1,mat,n,i,j,3)
        call mulmat(ld,temp1,sigma,temp2,n,nm,nm,1)
        call addmat(ld,temp2,temp3,temp3,n,nm,1)
        call xmat(ld,2,n,nm,temp3,v(1+n*nm))
*
* Compute derivatives of F3(a,lambda,x)
*
        call setmat(ld,0.d0,temp3,nm,nm)
        call rowcol(ld,temp3,ww,nm,i,j,1)
        call xmat(ld,2,nm,nm,temp3,v(1+2*n*nm))
      endif
*.......................................................................
*
* Derivatives with respect to SIGMAij
*
      if (der .eq. 3) then
*
* Compute derivatives of F1(a,lambda,x)
*
        call mulmat(ld,wi,mat,temp1,nm,n,n,1)
        call mulmat(ld,temp1,ww,temp2,nm,nm,n,1)
        call setmat(ld,0.d0,temp1,nm,nm)
        call rowcol(ld,temp1,temp2,nm,j,i,4)
        call mulmat(ld,temp1,ww,temp3,nm,n,nm,3)
*
        call mulmat(ld,ww,mat,temp1,nm,n,n,4)
        call setmat(ld,0.d0,temp2,nm,n)
        call rowcol(ld,temp2,temp1,n,i,j,1)
        call addmat(ld,temp2,temp3,temp3,nm,n,1)
        call xmat(ld,2,nm,n,temp3,v)
*
* Compute derivatives of F2(a,lambda,x)
*
        call setmat(ld,0.d0,temp1,n,nm)
        call rowcol(ld,temp1,wi,n,j,i,3)
        call mulmat(ld,temp1,wi,temp2,n,n,nm,1)
        call mulmat(ld,temp2,mat,temp1,n,n,n,1)
        call mulmat(ld,temp1,ww,temp3,n,nm,n,1)
*
        call mulmat(ld,mat,wi,temp1,n,nm,n,4)
        call setmat(ld,0.d0,temp2,n,nm)
        call rowcol(ld,temp2,temp1,n,j,i,4)
        call addmat(ld,temp2,temp3,temp3,n,nm,1)
        call xmat(ld,2,n,nm,temp3,v(1+n*nm))
*
* Compute derivatives of F3(a,lambda,x)
*
        call setmat(ld,0.d0,temp3,nm,nm)
        call xmat(ld,2,nm,nm,temp3,v(1+2*n*nm))
      endif
*.......................................................................
*
* Derivatives with respect to lambda
*
      if (der .eq. 4) then
*
        call addmat(ld,a,d,amd,n,n,3)
*
* Compute derivatives of F1(a,lambda,x)
*
        call mulmat(ld,wi,amd,temp1,nm,n,n,1)
        call mulmat(ld,temp1,ww,temp2,nm,nm,n,1)
        call mulmat(ld,temp2,sigma,temp1,nm,nm,nm,1)
        call mulmat(ld,temp1,ww,temp3,nm,n,nm,3)
*
        call mulmat(ld,sigma,ww,temp1,nm,n,nm,3)
        call mulmat(ld,temp1,amd,temp2,nm,n,n,3)
        call addmat(ld,temp2,temp3,temp3,nm,n,1)
*
        if (hom .ne. 1) call addmat(ld,f10,temp3,temp3,nm,n,1)
        call xmat(ld,2,nm,n,temp3,v)
*
* Compute derivatives of F2(a,lambda,x)
*
        call mulmat(ld,wi,sigma,temp1,n,nm,nm,2)
        call mulmat(ld,temp1,wi,temp2,n,n,nm,1)
        call mulmat(ld,temp2,amd,temp1,n,n,n,1)
        call mulmat(ld,temp1,ww,temp3,n,nm,n,1)
*
        call mulmat(ld,amd,wi,temp1,n,nm,n,4)
        call mulmat(ld,temp1,sigma,temp2,n,nm,nm,1)
        call addmat(ld,temp2,temp3,temp3,n,nm,1)
*
        if (hom .ne. 1) call addmat(ld,f20,temp3,temp3,n,nm,1)
        call xmat(ld,2,n,nm,temp3,v(1+n*nm))
*
* Compute derivatives of F3(a,lambda,x)
*
        call setmat(ld,0.d0,temp3,nm,nm)
        if (hom .ne. 1) call addmat(ld,f30,temp3,temp3,nm,nm,1)
        call xmat(ld,2,nm,nm,temp3,v(1+2*n*nm))
      endif
*
      return
      end
