      subroutine track1(ld,n,nm,m,l,nn,inp,trace,c1,c2,arcre,ansre,a,b,
     & c,v,r,y,iflag,ipivot,d,bvbt,ctrc,q,p,tau,ww,wi,sigma,f10,f20,f30,
     & temp1,temp2,temp3,aal,gc,go,work,p1,yold,yp,ypold,alpha,tz,w2,wp,
     & z0,z1,fvec,s1,qr,wa)
*
* Subroutine TRACK1 solves the optimal projection equations for the
* reduced order model problem. It is based on the normal flow algorithm
* for dense Jacobian matrices. The HOMPACK subroutine FIXPNF is used
* for that purpose.
*
* Input parameters:
* ld - leading dimension for all matrices except QR, at least n+1
* n,nm,m,l - dimensions of the system (A,B,C) and model (Am,Bm,Cm)
* nn - Jacobian dimension (2*n*nm + nm*nm)
* inp - strategy to be applied for choosing the starting point (1--5)
* trace - logical i/o unit for output
* c1,c2 - D is defined as D = -c1 * I + c2 * A
* arcre - relative error allowed of normal flow iterations along the curve
* ansre - relative error of the answer at lambda = 1
* a,b,c - system (A,B,C)
* v,r - white noise intensity V and weighting matrix R
* y - starting point (W,U,Sigma) if inp=5
*
* Output parameters:
* y - final point (lambda,W,U,Sigma)
* iflag - flag returned by FIXPNF
*
* Working storage:
* ipivot(nn+1) - integer working array
* d,bvbt,ctrc - D, B V B^t, C^t R C
* q,p,tau,ww,wi,sigma,f10,f20,f30,temp1,temp2,temp3,aal,gc,
*   go,work - working matrices of size (ld,ld)
* p1(16*ld*ld) - working array
* yold,yp,ypold - working arrays of size nn+1
* alpha,tz,w2,wp,z0,z1,fvec,s1 - working arrays of size nn+1
* qr(nn,nn+2) - working matrix
* wa(nn*(nn+13)/2) - working array
*
      integer hom,i,iflag,inp,ip(10),ipivot(nn+1),l,ld,m,n,nfe,nm,nn
      integer trace,type,z
      double precision a(ld,ld),aa,aal(ld,ld),alpha(nn+1),ansae,ansre
      double precision arclen,arcae,arcre,b(ld,ld),bvbt(ld,ld),c(ld,ld)
      double precision c1,c2,ctrc(ld,ld),d(ld,ld),f10(ld,ld),f20(ld,ld)
      double precision f30(ld,ld),fvec(nn+1),gc(ld,ld),go(ld,ld)
      double precision p(ld,ld),p1(16*ld*ld),q(ld,ld),qr(nn,nn+2)
      double precision r(ld,ld),s1(nn+1),sigma(ld,ld),sspar(8)
      double precision tau(ld,ld),temp1(ld,ld),temp2(ld,ld),temp3(ld,ld)
      double precision tz(nn+1),v(ld,ld),w2(nn+1),wa(nn*(nn+13)/2)
      double precision wi(ld,ld),work(ld,ld),wp(nn+1),ww(ld,ld),y(nn+1)
      double precision yold(nn+1),yp(nn+1),ypold(nn+1),z0(nn+1),z1(nn+1)
*
* Construct B V B^t and C^t R C, and D
*
      call bvrc(ld,n,m,l,b,c,bvbt,ctrc,v,r,temp1)
      call setd(ld,n,trace,c1,c2,a,d,temp1,temp2)
*
* Construct the starting point (W,U,Sigma)
*
      if ((nm .lt. m) .or. (nm .lt. l)) inp = 1
      if (inp .eq. 1) call initp1(ld,n,hom,ww,wi,sigma,temp1,ipivot)
      if (inp .eq. 2)
     &  call initp2(ld,n,nm,nn,c1,c2,d,bvbt,ctrc,y,hom,q,p,tau,
     &              f10,f20,f30,ww,wi,sigma,temp1,temp2,temp3,aal,
     &              qr,gc,go,work,wa,fvec,s1)
      if (inp .eq. 3)
     &  call initp3(ld,n,nm,m,l,nn,c1,c2,d,bvbt,ctrc,b,c,v,r,y,hom,q,p,
     &              tau,f10,f20,f30,ww,wi,sigma,temp1,temp2,temp3,
     &              qr,wa,fvec,s1)
      if (inp .eq. 4)
     &  call initp4(ld,n,nm,m,l,nn,c1,c2,d,bvbt,ctrc,y,hom,q,p,tau,
     &              f10,f20,f30,ww,wi,sigma,temp1,temp2,temp3,
     &              qr,wa,fvec,s1)
      if (inp .eq. 5) call initp5(ld,n,nm,nn,hom,ww,wi,sigma,y)
*
* Transform point from matrix form (W,U,Sigma) to array Y
*
      call xwus(ld,n,nm,2,ww,wi,sigma,y(2))
      y(1) = 0.d0
*
      if (trace .ne. 0) then
        write(trace,261)
261     format(/'y0')
        call writev(y,nn+1)
      endif
*
* Compute F(0,Y(0))=(F10,F20,F30)
*
      call f0s(ld,n,nm,ww,wi,sigma,f10,f20,f30,d,bvbt,ctrc,temp1,
     &         temp2,temp3)
*
* Set parameters for FIXPNF
*
      arcae = arcre
      ansae = ansre
      do 301 i = 1,8
        sspar(i) = 0.d0
301   continue
      iflag = -2
      type = 1
      hom = 0
*
* Prepare P1 and IP to be used in RHO and RHOJAC
*
      ip(1) = ld
      ip(2) = n
      ip(3) = nm
      ip(4) = type
      ip(5) = hom
      z = ld * ld
      call col2x(ld,ld,ld,a,p1(1))
      call col2x(ld,ld,ld,d,p1(z+1))
      call col2x(ld,ld,ld,bvbt,p1(2*z+1))
      call col2x(ld,ld,ld,ctrc,p1(3*z+1))
      call col2x(ld,ld,ld,f10,p1(4*z+1))
      call col2x(ld,ld,ld,f20,p1(5*z+1))
      call col2x(ld,ld,ld,f30,p1(6*z+1))
*
      if (trace .ne. 0) write(trace,200)
200   format(' ')
*
* Call tracking procedure
*
      do 311 i = 1,10
        call fixpnf(nn,y,iflag,arcre,arcae,ansre,ansae,trace,aa,nfe,
     &              arclen,yp,yold,ypold,qr,alpha,tz,ipivot,w2,wp,z0,z1,
     &              sspar,p1,ip)
        if (iflag .ne. 3) goto 401
311   continue
*
401   continue
       write(2,205) iflag, y(1)
 205   format(i2, /e20.10)
c     if (trace .ne. 0) write(trace,206)iflag, nfe, y(1)
c     write(6,206)iflag, nfe,y(1)
c206   format(/'iflag =',i2, '  total step = ', i6, ' lambda = ', e20.10)
*
      return
      end
