function [Ac,Bc,Cc,Dc,cost] = dfolqg(A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw)
% DFOLQG 	full-order discrete-time lqg compensator design
%
%        	[Ac,Bc,Cc,Dc,cost] = DFOLQG(A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw)
%       	returns the full-order discrete-time lqg controller 
%		for system (A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw) along with the 
%		H2-norm of the closed-loop system COST.  
%
%		NOTE: The state-space returned is not the same as formed 
%		by DREG.  DFOLQG uses the predictor form of the estimator
%		given by Franklin & Powell, as opposed to the 
%		current form.   See the Control System Toolbox User's 
%		manual and Franklin & Powell's "Digital Control of 
%		Dynamic Systems".
%
%       	see also DLQR, DLQE, DLYAP
%
	[ny,nu] = size(Dyu);
	[nx,nw] = size(Bw);

	R1 = Cz'*Cz;
	R2 = Dzu'*Dzu;
	R12 = Cz'*Dzu;

	V1 = Bw*Bw';
	V2 = Dyw*Dyw';
	V12 = Bw*Dyw';

	if R12 ~= 0*R12

		disp('WARNING: CROSS-WEIGHTING PRESENT');

	end

	[K,S] = dlqr(A,Bu,R1,R2,R12);
	
	if V12 ~= 0*V12

		disp('WARNING: NOISE CORRLELATION PRESENT');

	end

	L = dlqe(A,eye(nx),Cy,V1,V2,V12);

	Ac = A - Bu*K - L*Cy + L*Dyu*K;
	Bc = L;
	Cc = -K;
	Dc = zeros(nu,ny);

	[Acl,Bcl,Ccl,Dcl] = feedback(A,Bu,Cy,Dyu,Ac,Bc,Cc,Dc,1);

	Rtil = Ccl'*Ccl;
	Vtil = Bcl*Bcl';
	Qtil = dlyap(Acl, Vtil);

	cost = trace(Qtil*Rtil);
