function [Ac,Bc,Cc,Dc,cost] = folqg(A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw)
% FOLQG       	full-order continuous-time Linear-Quadratic
%		Gaussian (LQG) compensator design
%
%            	[Ac,Bc,Cc,Dc,cost] = FOLQG(A,Bu,Bw,Cy,Cz,Dyu,Dyw,
%	     	Dzu,Dzw) returns the full-order LQG controller 
%		for the TITO system given by {A,Bu,Bw,Cy,Cz,Dyu,
%		Dyw,Dzu,Dzw}.  COST is the H2-norm of the 
%		corresponding closed-loop transfer function.  
%
%             	see also LQR, LQE, REG, LYAP
%
	[nx,nw] = size(Bw);
	R1 = Cz'*Cz;
	R2 = Dzu'*Dzu;
	R12 = Cz'*Dzu;

	V1 = Bw*Bw';
	V2 = Dyw*Dyw';
	V12 = Bw*Dyw';

	if R12 ~= 0*R12

		disp('WARNING: CROSS-WEIGHTING PRESENT');

	end

	[K,S] = lqr2(A,Bu,R1,R2,R12);
	
	if V12 ~= 0*V12

		disp('WARNING: NOISE CORRLELATION PRESENT');
	end

	L = lqe2(A,eye(nx),Cy,V1,V2,V12);

	[Ac,Bc,Cc,Dc] = reg(A,Bu,Cy,Dyu,K,L);

	Cc = -1*Cc;

	atil = [A,Bu*Cc;Bc*Cy,Ac];

	btil = [Bw;Bc*Dyw];
	ctil = [Cz, Dzu*Cc];
	rtil = ctil'*ctil;
	vtil = btil*btil';
	qtil = lyap(atil, vtil);

	cost = trace(qtil*rtil);
