function C = kron2(a,b)

%====================================================================
%
% PURPOSE: This computes the kronecker product C = a (x) b, without
%          FOR loops.
%
% WRITTEN: L. Davis
%          29 October 1991
%          Harris Corporation
%
% REVISED: Richard S. Erwin
%          24 October 1993
%          University of Michigan
%
%====================================================================
   [na,ma] = size(a);
      [nb,mb] = size(b);
      nc = na*nb;  mc = ma*mb;
%
% Allocate room for C.
%
        C = zeros(nc,mc);
%
% Construct utility vectors.
%
 i = 1:nc;
       j = 1:mc;
       wrow = ones(1,nc);
      wcol = ones(1,mc);
      Nb = nb*wrow;
   Mb = mb*wcol;
%
% Find B = I (x) b.
%
      irow = rem((i-wrow),Nb) + wrow;
 icol = rem((j-wcol),Mb) + wcol;
 B = b(irow,icol);
%
% Find A = a (x) I.
%
  irow = ceil(i./Nb);
     icol = ceil(j./Mb);
     A = a(irow,icol);
%
% Find C = A * B.
%
    C = A.*B;
       end
