function [flag2] = pv_const(k,blk,nblk,kstart,gam,atilde,btilde,ctilde,bdtilde,cetilde)

%
% PV_CONST	determine if constraints of scaled Popov performance
%		are met.
%
%		[flag2] = pv_const(k,blk,nblk,kstart,gam,atilde,btilde,ctilde,
%		bdtilde,cetilde) checks to see if the constraints involved
%		in the scaled Popov performance are met by the data
%		contained in the arguments. 
%
%			FLAG2 = 1 := constraints have been violated
%			FLAG2 = 0 := constraints have been met
%			 
%

%
% last modified 12/6/95 RSE
%


          cum = cumsum(blk(:,1));
          scum = cumsum(blk(:,1).*blk(:,1));

          wvec = k(kstart:kstart+scum(nblk)-1);
          zvec = k(kstart+scum(nblk):kstart+2*scum(nblk)-1);

          WW = [];
          ZZ = [];

          for i = 1:nblk,

              if blk(i,1) == 1,

    		WW = daug(WW,wvec(scum(i)));
   	 	ZZ = daug(ZZ,zvec(scum(i)));

   	      elseif blk(i,1) == 2,

    		WW = daug(WW,ivec(wvec((scum(i)-3):scum(i)),2,2));
    		ZZ = daug(ZZ,ivec(zvec((scum(i)-3):scum(i)),2,2));

              end

	  end

      invGam = inv(gam*ZZ - WW*cetilde*bdtilde - bdtilde'*cetilde'*WW);
      Ahat   = atilde - gam^(-1)*bdtilde*cetilde;
      XXhat  = cetilde'*ZZ + Ahat'*cetilde'*WW;

%
%  form hamiltonian for Ptilde Riccatti equation:
%
%	A'X + X*A + X*R*X - Q = 0
%
%  hamiltonian is:
%
%	H = [ A     R ]
%           [ Q    -A']
%

      clear A Q R

      A = Ahat + bdtilde*invGam*XXhat';
      R = bdtilde*invGam*bdtilde';
      Q = -(ctilde'*ctilde + XXhat*invGam*XXhat');

      H = [A R; Q -A'];

      [P1,P2,flag2] = ric_schr(H);
      ptilde = P2/P1;

      peval = max(min(real(eig(ptilde))) <= 0);
      qeval = max(min(real(eig(ZZ))) <= 0);
      consteval = max(min(real(eig(invGam))) <= 0);
	  
      flag2 = flag2 + peval + qeval + consteval;

      if flag2 ~= 0 

		fprintf(1,'%c','ATTEMPTED POPOV CONSTRAINT VIOLATION')
		fprintf(1,'%c\n',' ')

      end

	  
