function [W,Z,fail] = pv_lmi(a,b,c,r,gamma,blk)

% PV_LMI	finds initializing W and Z multipliers for
%		the scaled Popov performance problem.
%
%		[W,Z,fail] = pv_lmi(a,b,c,r,gamma,blk) finds 
%		the stabilizing solution P to the scaled Popov 
%		Riccati equation by minimizing tr P using the 
%		matrix variables W and Z subject to
% 
%  | (A-BC/gamma)'P+P(A-BC/gamma)+R  PB+C'Z+(A-BC/gamma)'C'W |
%  |                                                         |  <  0
%  |   B'P + ZC + WC(A-BC/gamma)      WCB + B'C'W - gamma Z  |
%
%                               P  >  0 
%                         
%                               Q  >  0
%
%==============================================================
% 
% REVISED: R. Scott Erwin
%	   December 6 1995
%          University of Michigan
%
%==============================================================
%
% determine sizes of matrices

	ns = size(a,1);
	nu = size(b,2);
%
%
	abcg = a - b*c/gamma;

	if max(real(eig(abcg))) < 0
%
%
		setlmis([]);
%
		[p] = lmivar(1,[ns 1]);
		[w] = lmivar(1,blk);
		[z] = lmivar(1,blk);
%
		[lmi1] = newlmi;
		lmiterm([lmi1 1 1 p],1,abcg,'s');
		lmiterm([lmi1 1 1 0],r);
		lmiterm([lmi1 2 1 p],b',1);
		lmiterm([lmi1 2 1 w],1,c*abcg);
		lmiterm([lmi1 2 1 z],1,c);
		lmiterm([lmi1 2 2 w],1,c*b,'s');
		lmiterm([lmi1 2 2 z],-gamma,1);
%
		[lmi2] = newlmi;
		lmiterm([-lmi2 1 1 p],1,1);
%
		[lmi3] = newlmi;
		lmiterm([-lmi3 1 1 z],1,1);
%
		rlmi = getlmis;

		[tmin,xfeas] = feasp(rlmi,[0 100 0 0 0]);

		if tmin > 0

			P = dec2mat(rlmi,xfeas,p);
			W = dec2mat(rlmi,xfeas,w);
			Z = dec2mat(rlmi,xfeas,z);

			stab = a - b*c/gamma + b*inv(gamma*Z - W*c*b - b'*c'*W)*(b'*P+Z*c+W*c*(a - b*c/gamma));

			if max(real(eig(stab))) < 0;

				setlmis(rlmi);

				tflag = 1;
  				lmiterm([-lmi1 1 1 0],max(0,1.1*tmin)); 
  				lmiterm([-lmi1 2 2 0],max(0,1.1*tmin));

				rlmi = getlmis;


			else

				tflag = 0;
				
			end

		else 
		
			tflag = 1;

		end


		if tflag == 1

			setlmis(rlmi);
	
  			cmat = mat2dec(rlmi,eye(ns),zeros(nu,nu),zeros(nu,nu));

  			[copt,xopt] = mincx(rlmi,cmat,[1e-6 100 0 0 0],xfeas);

  			P = dec2mat(rlmi,xopt,p);
  			W = dec2mat(rlmi,xopt,w);
  			Z = dec2mat(rlmi,xopt,z);

  			stab = a - b*c/gamma + b*inv(gamma*Z - W*c*b - b'*c'*W)*(b'*P+Z*c+W*c*(a - b*c/gamma));
 
 			if max(real(eig(stab))) < 0

    				fail = 0;
  			else

    				P = [];
    				W = [];
    				Z = [];
    				fail = 1;

  			end

		else

 	 		P = [];
  			W = [];
  			Z = [];
  			fail = 1;

		end

	else

		W = [];
		Z = [];
		fail = 1;

	end
